@@ -181,6 +181,7 @@
 //        GMD_CLOSEDHIHAT        42       Closed Hi-Hat          
 //        GMD_HIGHFLOORTOM       43       High Floor Tom         
 //        GMD_PEDALHIHAT         44       Pedal Hi-Hat           
+//        GMD_LOWTOM             45       Low Tom
 //        GMD_OPENHIHAT          46       Open Hi-Hat            
 //        GMD_LOWMIDTOM          47       Low-Mid Tom            
 //        GMD_HIMIDTOM           48       Hi-Mid Tom             
@@ -368,6 +369,7 @@
 #define GMD_CLOSEDHIHAT        42   
 #define GMD_HIGHFLOORTOM       43   
 #define GMD_PEDALHIHAT         44   
+#define GMD_LOWTOM             45
 #define GMD_OPENHIHAT          46   
 #define GMD_LOWMIDTOM          47   
 #define GMD_HIMIDTOM           48   
