@@ -46,7 +46,7 @@
 #define DRIVER_PA_UNIX_OSS 12
 #define DRIVER_PA_WIN_WMME 13
 #define DRIVER_PA_WIN_DS   14
-#define DRIVER_MAC_CARBON  15
+#define DRIVER_COREAUDIO   15
 #define DRIVER_END         16
 
 /****************************************************************/
@@ -67,7 +67,7 @@ void printaudiohelp(void)
   printf("       [-aout linux]       soundcard under Linux (OSS).\n");
   printf("       [-aout pa_win_wmme] soundcard under Windows (WMME API).\n");
   printf("       [-aout pa_win_ds]   soundcard under Windows (Direct API).\n");
-  printf("       [-aout mac_carbon]  soundcard under Mac OS X Carbon.\n");
+  printf("       [-aout coreaudio]   soundcard under Mac OS X (CoreAudio).\n");
   printf("       [-aout freebsd]     soundcard under FreeBSD.\n");
   printf("       [-aout irix]        soundcard under IRIX (6.2 or later).\n");
   printf("       [-aout hpux]        soundcard under HPUX.\n");
@@ -85,7 +85,7 @@ void printaudiohelp(void)
   printf("       [-ain linux]        soundcard under Linux (OSS).\n");
   printf("       [-ain pa_win_wmme]  soundcard under Windows (WMME API).\n");
   printf("       [-ain pa_win_ds]    soundcard under Windows (Direct API).\n");
-  printf("       [-ain mac_carbon]   soundcard under Mac OS X Carbon.\n");
+  printf("       [-ain coreaudio]    soundcard under Mac OS X (CoreAudio).\n");
   printf("       [-ain freebsd]      soundcard under FreeBSD.\n");
   printf("       [-ain irix]         soundcard under IRIX (6.2 or later).\n");
   printf("       [-ain hpux]         soundcard under HPUX.\n");
@@ -100,6 +100,8 @@ void printaudiohelp(void)
 int aoutfilecheck(char * fname)
 
 {
+  int i, found;
+
   aoutname = dupval(fname);
 
   /* first, non-filename entries */
@@ -188,19 +190,36 @@ int aoutfilecheck(char * fname)
 	}
       return 0;
     }
-  if (!strcmp(fname,"mac_carbon"))
+  if (!strcmp(fname,"coreaudio"))
     {
-      aout = DRIVER_MAC_CARBON;
+      aout = DRIVER_COREAUDIO;
       aoutflow = ACTIVE_FLOW;
 
-      if (ain != DRIVER_MAC_CARBON)
+      found = 0;
+
+      for (i = 2; i < sfront_argc; i++)
+	if (!strcmp("-cin", sfront_argv[i-1]))
+	  {
+	    found = !strcmp("coremidi", sfront_argv[i]);
+	    break;
+	  }
+
+      if (ain != DRIVER_COREAUDIO)
 	{
-	  printf("Macintosh OS X Carbon notes:\n");
-	  printf("[1] cc -O3 -I/Developer/Headers/FlatCarbon sa.c\n");
-	  printf("    -framework Carbon -o sa\n");
-	  printf("[2] Might also work under Mac OS 9\n");
-	  printf("[3] Thanks to portaudio.com.\n\n");
+	  printf("Macintosh OS X CoreAudio notes:\n");
+
+	  if (found)
+	    {
+	      printf("[1] cc -O3 sa.c -lm -framework coreaudio" 
+		     " -framework coremidi \\\n");
+	      printf("    -framework corefoundation -o sa\n");
+	    }
+	  else
+	    printf("[1] cc -O3 sa.c -lm -framework CoreAudio -o sa\n");
+
+	  printf("\n");
 	}
+
       return 0;
     }
   if (DEPRECIATED_DSOUND_TESTING && (!strcmp(fname,"dsound")))
@@ -349,19 +368,16 @@ int ainfilecheck(char * fname)
 	}
       return 0;
     }
-  if (!strcmp(fname,"mac_carbon"))
+  if (!strcmp(fname,"coreaudio"))
     {
-      ain = DRIVER_MAC_CARBON;
+      ain = DRIVER_COREAUDIO;
       ainflow = ACTIVE_FLOW;
       ainlatency = LOW_LATENCY_DRIVER; 
 
-      if (aout != DRIVER_MAC_CARBON)
+      if (aout != DRIVER_COREAUDIO)
 	{
-	  printf("Macintosh OS X Carbon notes:\n");
-	  printf("[1] cc -O3 -I/Developer/Headers/FlatCarbon sa.c\n");
-	  printf("    -framework Carbon -o sa\n");
-	  printf("[2] Might also work under Mac OS 9\n");
-	  printf("[3] Thanks to portaudio.com.\n\n");
+	  printf("Macintosh OS X CoreAudio notes:\n");
+          printf("[1] cc -O3 sa.c -lm -framework CoreAudio -o sa\n\n");
 	}
       return 0;
     }
@@ -413,7 +429,7 @@ int makeaudiotypeout(int anum)
   case DRIVER_PA_UNIX_OSS:
   case DRIVER_PA_WIN_WMME:
   case DRIVER_PA_WIN_DS:
-  case DRIVER_MAC_CARBON:
+  case DRIVER_COREAUDIO:
   case DRIVER_VCDAT:
     ret = SAMPLE_FLOAT;
     break;
@@ -436,7 +452,7 @@ int makeaudiotypein(int anum)
   case DRIVER_PA_UNIX_OSS:
   case DRIVER_PA_WIN_WMME:
   case DRIVER_PA_WIN_DS:
-  case DRIVER_MAC_CARBON:
+  case DRIVER_COREAUDIO:
   case DRIVER_VCDAT:
     ret = SAMPLE_FLOAT;
     break;
@@ -462,7 +478,7 @@ int makeaoutsync(int anum)
   case DRIVER_PA_UNIX_OSS:
   case DRIVER_PA_WIN_WMME:
   case DRIVER_PA_WIN_DS:
-  case DRIVER_MAC_CARBON:
+  case DRIVER_COREAUDIO:
     ret = 1;
     break;
   default:               /* use standard UNIX version */
@@ -485,7 +501,7 @@ int makeainsync(int anum)
   case DRIVER_PA_UNIX_OSS:    /* only here as an example, may not work */
   case DRIVER_PA_WIN_WMME:
   case DRIVER_PA_WIN_DS:
-  case DRIVER_MAC_CARBON:
+  case DRIVER_COREAUDIO:
     ret = 1;
     break;
   default:               
@@ -513,7 +529,7 @@ int makeaouttimedefault(int anum)
   case DRIVER_PA_UNIX_OSS:
   case DRIVER_PA_WIN_WMME:
   case DRIVER_PA_WIN_DS:
-  case DRIVER_MAC_CARBON:
+  case DRIVER_COREAUDIO:
     ret = PLAYBACK;
     break;
   }
@@ -540,7 +556,7 @@ int makeaintimedefault(int anum)
   case DRIVER_PA_UNIX_OSS:
   case DRIVER_PA_WIN_WMME:
   case DRIVER_PA_WIN_DS:
-  case DRIVER_MAC_CARBON:
+  case DRIVER_COREAUDIO:
     ret = PLAYBACK;
     break;
   }
@@ -607,7 +623,7 @@ void makeainparams(int anum, int * srate, int * inchannels)
   case DRIVER_PA_UNIX_OSS:
   case DRIVER_PA_WIN_WMME:
   case DRIVER_PA_WIN_DS:
-  case DRIVER_MAC_CARBON:
+  case DRIVER_COREAUDIO:
     /* modern soundcards optimized to this configuration */
     *inchannels = 2;
     *srate = 44100;
@@ -729,9 +745,8 @@ void makeaudiodriver(int anum)
     makeportaudiocommon();
     makepa_win_wmme();
     break;
-  case DRIVER_MAC_CARBON:
-    makeportaudiocommon();
-    makepa_mac();
+  case DRIVER_COREAUDIO:
+    makecoreaudio();
     break;
   case DRIVER_PA_WIN_DS:
     makeportaudiocommon();
