@@ -28,38 +28,46 @@
 /*             instr code generation routines                   */
 /****************************************************************/
 
+
 /****************************************************************/
 /*             service routine: for updating nextstate          */
 /*       makes sure deleted instruments aren't effects          */
 /****************************************************************/
 
-void nextstateupdate(void)
+void nextstateupdate(int * lcptr)
 
 {
+  int lc;
+
+  lc = lcptr ? (*lcptr) : 0;
+
   /* note a table memory leak happens if ALLDONE is set. this */
   /* can be avoided by looping through MAXTABLESTATE tables,  */
   /* or by storing table width in iline.                      */
 
-  fprintf(outfile,"    oldstate = nextstate;\n");
-  fprintf(outfile,"    nextstate = (nextstate+1) %% MAXSTATE;\n");
-  fprintf(outfile,"    while ((oldstate != nextstate) && \n");
-  fprintf(outfile,"           (ninstr[nextstate].iline != NULL))\n");
-  fprintf(outfile,"      nextstate = (nextstate+1) %% MAXSTATE;\n");
-  fprintf(outfile,"    if (oldstate == nextstate)\n");
-  fprintf(outfile,"    {");
-  fprintf(outfile,"      nextstate = (nextstate+1) %% MAXSTATE;\n");
-  fprintf(outfile,"      while ((oldstate != nextstate) && \n");
-  fprintf(outfile,"          (ninstr[nextstate].iline->time == 0.0F) &&\n");
-  fprintf(outfile,"          (ninstr[nextstate].iline->noteon == PLAYING))\n");
-  fprintf(outfile,"        nextstate = (nextstate+1) %% MAXSTATE;\n");
-  fprintf(outfile,"      ninstr[nextstate].iline->noteon = ALLDONE;\n");
-  fprintf(outfile,"      ninstr[nextstate].iline = NULL;\n");
-  fprintf(outfile,"    }");
+  z[lc++]="    oldstate = nextstate;";
+  z[lc++]="    nextstate = (nextstate+1) % MAXSTATE;";
+  z[lc++]="    while ((oldstate != nextstate) && ";
+  z[lc++]="           (ninstr[nextstate].iline != NULL))";
+  z[lc++]="      nextstate = (nextstate+1) % MAXSTATE;";
+  z[lc++]="    if (oldstate == nextstate)";
+  z[lc++]="    {";
+  z[lc++]="      nextstate = (nextstate+1) % MAXSTATE;";
+  z[lc++]="      while ((oldstate != nextstate) && ";
+  z[lc++]="          (ninstr[nextstate].iline->time == 0.0F) &&";
+  z[lc++]="          (ninstr[nextstate].iline->noteon == PLAYING))";
+  z[lc++]="        nextstate = (nextstate+1) % MAXSTATE;";
+  z[lc++]="      ninstr[nextstate].iline->noteon = ALLDONE;";
+  z[lc++]="      ninstr[nextstate].iline = NULL;";
+  z[lc++]="    }";
 
+  if (lcptr)
+    *lcptr = lc;
+  else
+    printraw(lc);
 }
 
 
-
 /****************************************************************/
 /*       prints NV/NVI/NG/NGI as appropriate for variable       */
 /****************************************************************/
@@ -333,10 +341,9 @@ void extendcode(tnode * tbranch)
 
   if (currinstance && (currinstance == outputbusinstance))
     {      
-      printf("Error: extend statement not permitted in instr" 
-	     " sent output_bus.\n\n");
-      showerrorplace(tbranch->down->linenum, 
-		     tbranch->down->filename);
+      warningmessage(tbranch->down, "extend statement not executed in instr"
+		     " sent the output bus");
+      return;
     }
 
   fprintf(outfile,"  if ((NS(iline->sdur) < 0.0F)||\n");
@@ -782,7 +789,7 @@ void instrcode(tnode * tbranch)
   fprintf(outfile,"    while (d_%snext < d_%send)\n",val,val);
   fprintf(outfile,"     {\n");
   fprintf(outfile,"       if ((d_%snext->noteon == ALLDONE)||\n",val);
-  fprintf(outfile,"           (d_%snext->endtime == 0.0F))\n",val); 
+  fprintf(outfile,"           (d_%snext->noteon == NOTUSEDYET))\n",val); 
   fprintf(outfile,"        break;\n");
   fprintf(outfile,"       d_%snext++;\n",val);
   fprintf(outfile,"     }\n");
@@ -850,7 +857,7 @@ void instrcode(tnode * tbranch)
 	  ,val,val);
   fprintf(outfile,"    d_%snext->kbirth = kcycleidx;\n",val);
   fprintf(outfile,"    d_%snext->noteon = PLAYING;\n",val);
-  nextstateupdate();
+  nextstateupdate(NULL);
   fprintf(outfile,"    if (pass == KPASS)\n");
   fprintf(outfile,"     {\n");
   fprintf(outfile,"      pass = IPASS;\n");
@@ -968,16 +975,14 @@ void opcallcode(tnode * tptr)
 
   if (currarrayindex == 0)
     {
-      if ((tptr->optr->rate == currblockrate) || tptr->special || 
-	  polyopcallexcept(tptr))
+      if ((tptr->optr->rate == currblockrate) || polyopcallexcept(tptr))
 	{
 	  opcallprint(tptr, SAMERATETYPE);
 	}
       else
 	{
-
-	  /* opcalls at slower rate */
-	  /* left to do: currintprint, rethink tptr->special above */
+	  /* opcalls at slower rate   */
+	  /* left to do: currintprint */
 
 	  sptr = tptr->optr->sptr;
 
@@ -1185,9 +1190,6 @@ void blocktree(tnode * tbranch, int mode)
   while (tptr != NULL)
     if ((tptr->ttype == S_STATEMENT))
       {
-	if ((mode == PRINTAPASS) && (tptr->rate == KRATETYPE) && 
-	    (tptr->special == 1))
-	  specialcode(tptr);
 	if ((tptr->down->ttype == S_INSTR) && (mode == PRINTIPASS)
 	    && (tptr->rate == KRATETYPE))
 	  instrstubcode(tptr);
@@ -1236,6 +1238,8 @@ void blocktree(tnode * tbranch, int mode)
 	    blocktree(tptr->down,PRINTTOKENS);
 	    break;
 	  }
+	if ((mode == PRINTAPASS) && (tptr->special == 1))
+	  specialcode(tptr);
 	tptr = tptr->next;
       }
     else
