@@ -71,7 +71,7 @@ void makecontrolsys(void)
       z[lc++]="  else";
       z[lc++]="   {";
       z[lc++]="     ne = csys_midievent(&cmd,&ndata,&vdata,&ec,&fval);";
-      z[lc++]="     if ((cmd & 0xF0) > CSYS_MIDI_SPECIAL)";
+      z[lc++]="     if (cmd >= CSYS_MIDI_TSTART)";
       z[lc++]="       nsys_midisend(cmd, ndata, vdata, ec);";
       z[lc++]="   }";
     }
@@ -215,6 +215,17 @@ void makecontrolsys(void)
       z[lc++]="            sysidx->notenum += 256;";
       z[lc++]="         }";
       z[lc++]="      break;";
+      z[lc++]="     case CSYS_MIDI_CC_RESETALLCONTROL:";
+      z[lc++]="      for (i = 0; i < CSYS_MIDI_CC_ALLSOUNDOFF; i++)";
+      z[lc++]="        NG(CSYS_FRAMELEN*(ec+CSYS_EXTCHANSTART)+CSYS_CCPOS+i)=0.0F;";
+      z[lc++]="      NG(CSYS_FRAMELEN*(ec + CSYS_EXTCHANSTART)+";
+      z[lc++]="         CSYS_MIDI_CC_CHANVOLUME_MSB) = 100.0F;";
+      z[lc++]="      NG(CSYS_FRAMELEN*(ec + CSYS_EXTCHANSTART)+";
+      z[lc++]="         CSYS_MIDI_CC_PAN_MSB) = 64.0F;";
+      z[lc++]="      NG(CSYS_FRAMELEN*(ec + CSYS_EXTCHANSTART)+";
+      z[lc++]="         CSYS_MIDI_CC_EXPRESSION_MSB) = 127.0F;";
+      z[lc++]="      csys_bank = csys_banklsb = csys_bankmsb = 0;";
+      z[lc++]="      break;";
       z[lc++]="     default:";
       z[lc++]="      break;";
       z[lc++]="     }";
