@@ -133,7 +133,8 @@ void co_constcollapse(tnode * tptr)
 void co_identcollapse(tnode * tptr) 
 
 {
-  if (standardname(tptr->down))
+  if (standardname(tptr->down) && strcmp(tptr->down->val,"inchan") &&
+      strcmp(tptr->down->val,"outchan"))
     {
       tptr->vol = standardcollapse(tptr->down);
       if (tptr->vol == CONSTANT)
