@@ -461,17 +461,19 @@ void coreopcodedoinline(tnode * ident)
     if (!(strcmp(ident->val,"ftlen")))
       {
 	printinlinetable(tptr);
-	fprintf(outfile,".len");
+	fprintf(outfile,".lenf");
       }
     if (!(strcmp(ident->val,"ftloop")))
       {
+	fprintf(outfile,"((float)(");
 	printinlinetable(tptr);
-	fprintf(outfile,".start");
+	fprintf(outfile,".start))");
       }
     if (!(strcmp(ident->val,"ftloopend"))) 
       {
+	fprintf(outfile,"((float)(");
 	printinlinetable(tptr);
-	fprintf(outfile,".end");
+	fprintf(outfile,".end))");
       }
     if (!(strcmp(ident->val,"ftsetbase")))
       return;
