@@ -969,6 +969,10 @@ int yylex (void)
 	break;
       }
 
+      if (cin || session)
+	if (addvsym(&mpegtokens, buf, S_IDENT) == INSTALLED)
+	  mpegtokens->width = mpegtokencount++;
+
       yylval->ttype = lexttl = S_IDENT;
       return IDENT;
     }
