@@ -166,7 +166,7 @@ void tgen_cubicseg(tableinfo * ti, int pnum, float * p)
 
 {
   int i, j, len, endpoint;
-  float Q, R, S, T, a, b, c, d;
+  float Q, R, S, T, a, b, c, d, xf;
 
   if (pnum < 7)
     epr(0,"control driver","tgen_cubicseg","Too few parameters");
@@ -201,6 +201,8 @@ void tgen_cubicseg(tableinfo * ti, int pnum, float * p)
     }
 
   i = 0;
+  xf = 0.0F;
+
   for (j = 1; j < pnum; j += 4)
     {
       endpoint = p[j+4];
@@ -238,8 +240,8 @@ void tgen_cubicseg(tableinfo * ti, int pnum, float * p)
     
       while (i < endpoint)
 	{
-	  ti->t[i] =  i*i*i*a + i*i*b + i*c + d;
-	  i++;
+	  ti->t[i] =  xf*xf*xf*a + xf*xf*b + xf*c + d;
+	  xf = (++i);
 	}
     }
   ti->t[ti->len] = ti->t[0];
@@ -724,7 +726,7 @@ void tgen_spline(tableinfo * ti, int pnum, float * p)
 
 {
   int i, j, len, endpoint, newpoint;
-  float Q, R, S, a, b, c, d, k1, k2;
+  float Q, R, S, a, b, c, d, k1, k2, xf;
 
   if (pnum < 6)
     epr(0,"control driver","tgen_spline","Too few parameters");
@@ -759,6 +761,8 @@ void tgen_spline(tableinfo * ti, int pnum, float * p)
     }
 
   endpoint = i = 0;
+  xf = 0.0F;
+
   for (j = 1; (j+4) < pnum; j += 3)
     {
 
@@ -801,8 +805,8 @@ void tgen_spline(tableinfo * ti, int pnum, float * p)
 
 	  while (i < endpoint)
 	    {
-	      ti->t[i] =  i*i*i*a + i*i*b + i*c + d;
-	      i++;
+	      ti->t[i] =  xf*xf*xf*a + xf*xf*b + xf*c + d;
+	      xf = (++i);
 	    }
 	}
     }
