@@ -59,6 +59,20 @@
 
 #define CSYSI_BUFFSIZE   1024  /* size of the holding buffer   */
 #define CSYSI_SENDSIZE   256   /* maximum size of one packet[] */
+#define CSYSI_SYSEXSIZE    128  /* buffer size for F0/F4/F5     */
+
+/****************************/
+/* System command constants */
+/****************************/
+
+#define CSYSI_BLOCK_SYSTEM_COMMANDS   0   /* remove 0xF from source */
+#define CSYSI_PASS_SYSTEM_COMMANDS    1   /* retain 0xF             */
+
+#define CSYSI_SYSTEM_GATE CSYSI_PASS_SYSTEM_COMMANDS  /* sets mode */
+
+#define CSYSI_COMMAND_ONGOING         0   /* arbitrary-length command ongoing   */
+#define CSYSI_COMMAND_COMPLETED       1   /* arbitrary-length command completed */
+
 
 /**********************/
 /* preamble constants */
@@ -68,7 +82,7 @@
 
  0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 
 -------------------------------------------------------------------
-|    0xFD       |T|R|R|R|  EXT  |  tstamp (T = 1)  ... | MIDI ... |
+|    0xF7       |T|R|R|R|  EXT  |  tstamp (T = 1)  ... | MIDI ... |
 -------------------------------------------------------------------
 
   T is set to 1 if a non-NOW timestamp follows the header octet.
@@ -84,13 +98,30 @@
 #define CSYSI_PREAMBLE_POS_CONTROL 1  /* second octet holds control bits     */
 #define CSYSI_PREAMBLE_POS_TSTAMP  2  /* last 4 (optional) octets for tstamp */
 
-#define CSYSI_PREAMBLE_FLAGS_MARKER  0xFD  /* marker value: for easy parsing */
-#define CSYSI_PREAMBLE_FLAGS_T       0x80  /* timestamp follows this octet   */
-#define CSYSI_PREAMBLE_FLAGS_R2      0x40  /* reserved bits                  */
-#define CSYSI_PREAMBLE_FLAGS_R1      0x20  
-#define CSYSI_PREAMBLE_FLAGS_R0      0x10  
-#define CSYSI_PREAMBLE_FLAGS_EXT     0x0F  /* the EXT nibble                 */
- 
+#define CSYSI_PREAMBLE_FLAGS_MARKER  0xF7u  /* marker value: for easy parsing */
+#define CSYSI_PREAMBLE_FLAGS_T       0x80u  /* timestamp follows this octet   */
+#define CSYSI_PREAMBLE_FLAGS_R2      0x40u  /* reserved bits                  */
+#define CSYSI_PREAMBLE_FLAGS_R1      0x20u  
+#define CSYSI_PREAMBLE_FLAGS_R0      0x10u  
+#define CSYSI_PREAMBLE_FLAGS_EXT     0x0Fu  /* the EXT nibble                 */
+
+/*****************************/
+/* simulated F4/F5 constants */
+/*****************************/
+
+/*
+  SysEx format to simulate undefined commands:
+
+  | F7 | 7D | code | octone (if bit set) | octtwo (if bit set) | F0 |
+
+*/
+
+#define CSYSI_UNDEFINED_ID          0x7Du  /* Manufacturers ID */
+
+#define CSYSI_UNDEFINED_F4          0x40u   /* set is F4, clear is F5 */
+#define CSYSI_UNDEFINED_OCTONE      0x20u   /* presence of 1st octet  */
+#define CSYSI_UNDEFINED_OCTTWO      0x10u   /* presence of 2nd octet  */   
+
 /************************/
 /* input port variables */
 /************************/
@@ -99,10 +130,16 @@ MIDIClientRef csysi_inclient;  /* the input client  */
 MIDIPortRef   csysi_inport;    /* it's port         */
 
 typedef struct csysi_portinfo {
-  int ext;               /* EXT field for preamble, also array index */
-  int valid;             /* 0 if unused, 1 if online, -1 if offline  */
-  MIDIEndpointRef src;   /* current src  (may be invalid if offline) */
-  int id;                /* unique ID of src (currently unused)      */
+  int ext;                 /* EXT field for preamble, also array index */
+  int valid;               /* 0 if unused, 1 if online, -1 if offline  */
+  MIDIEndpointRef src;     /* current src  (may be invalid if offline) */
+  int id;                  /* unique ID of src (currently unused)      */
+
+#if (CSYSI_SYSTEM_GATE == CSYSI_PASS_SYSTEM_COMMANDS)
+  unsigned char sysbuff[CSYSI_SYSEXSIZE];       /* System command buffer */
+  int sysex_size;           /* current size of sysbuff[]                 */
+#endif
+
 } csysi_portinfo;
 
 csysi_portinfo csysi_ports[CSYSI_MAXPORTS];
@@ -146,6 +183,12 @@ extern void csysi_read_proc(const MIDIPacketList * pktlist,
 extern void csysi_notify_proc(const MIDINotification * message,
 			      void * refcon);
 
+extern int csysi_bugrecovery(MIDIPacket * p, unsigned char * mdata,
+			     int * idx, int len);
+
+extern int csysi_packet_arblength(MIDIPacket * p, unsigned char * mdata,
+				  int * idx, int * len, unsigned char ext);
+
 /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
 /*     high-level functions: called by sfront engine            */
 /*______________________________________________________________*/
@@ -397,24 +440,64 @@ int csys_midievent(unsigned char * cmd,   unsigned char * ndata,
 
   /* assumes:
    *   1 - integral number of commands
-   *   2 - all system commands filtered out except 0xFF
-   *   3 - input stream is perfect MIDI
+   *   2 - F0, F7, "specialized"
+   *   3 - F4/F5 has 0, 1, or 2 data bytes
+   *   4 - input stream is perfect MIDI + "specials"
    */
 
-  if (csysi_buffer[csysi_cnt] != CSYS_MIDI_SYSTEM_RESET)
+  if (csysi_buffer[csysi_cnt] < (unsigned char)(CSYS_MIDI_SYSTEM))
     {
-      *cmd = 0xF0 & csysi_buffer[csysi_cnt];
-      *extchan = (0x0F & csysi_buffer[csysi_cnt++]) + csysi_currext;
-      *ndata = csysi_buffer[csysi_cnt++];
-
-      if ((*cmd != CSYS_MIDI_PROGRAM) && (*cmd != CSYS_MIDI_CTOUCH))
-	*vdata = csysi_buffer[csysi_cnt++];
+      if ((CSYSI_SYSTEM_GATE == CSYSI_PASS_SYSTEM_COMMANDS) &&
+	  (csysi_buffer[csysi_cnt] < (unsigned char) (CSYS_MIDI_NOTEOFF)))
+	{
+	  *cmd = csysi_buffer[csysi_cnt++];
+	  *extchan = csysi_currext;
+	  *ndata = csysi_buffer[csysi_cnt++];
+	  
+	  if (*cmd != CSYS_MIDI_GMRESET)
+	    *vdata = csysi_buffer[csysi_cnt++];
+	}
+      else
+	{
+	  *cmd = 0xF0u & csysi_buffer[csysi_cnt];
+	  *extchan = (0x0Fu & csysi_buffer[csysi_cnt++]) + csysi_currext;
+	  *ndata = csysi_buffer[csysi_cnt++];
+	  
+	  if ((*cmd != CSYS_MIDI_PROGRAM) && (*cmd != CSYS_MIDI_CTOUCH))
+	    *vdata = csysi_buffer[csysi_cnt++];
+	}
     }
   else
     {
-      *cmd = CSYS_MIDI_POWERUP;
+      *cmd = csysi_buffer[csysi_cnt++];
       *extchan = csysi_currext;
-      csysi_cnt++;
+
+      if (*cmd < CSYS_MIDI_SYSTEM_SYSEX_END)
+	{
+	  if ((*cmd == CSYS_MIDI_SYSTEM_QFRAME) || 
+	      (*cmd == CSYS_MIDI_SYSTEM_SONG_SELECT) ||
+	      (*cmd == CSYS_MIDI_SYSTEM_SONG_PP))
+	    *ndata = csysi_buffer[csysi_cnt++];
+
+	  if (*cmd == CSYS_MIDI_SYSTEM_SONG_PP)
+	    *vdata = csysi_buffer[csysi_cnt++];
+
+	  if ((*cmd == CSYS_MIDI_SYSTEM_UNUSED1) || 
+	      (*cmd == CSYS_MIDI_SYSTEM_UNUSED2))
+	    {
+	      if ((csysi_cnt < csysi_len) && 
+		  !(csysi_buffer[csysi_cnt] & 0x80u))
+		*ndata = csysi_buffer[csysi_cnt++];
+	      else
+		*ndata = CSYS_MIDI_SYSTEM_SYSEX_END;
+
+	      if ((csysi_cnt < csysi_len) && 
+		  !(csysi_buffer[csysi_cnt] & 0x80u))
+		*vdata = csysi_buffer[csysi_cnt++];
+	      else
+		*vdata = CSYS_MIDI_SYSTEM_SYSEX_END;
+	    }
+	}
     }
 
   if (csysi_cnt == csysi_len)
@@ -441,7 +524,7 @@ void csys_shutdown(void)
 
 extern int csysi_packet_shorten(MIDIPacket * p, 
 				unsigned char * mdata,
-				int * idx); 
+				int * idx, unsigned char ext); 
 
 /****************************************************************/
 /*                  callback for MIDI input                     */
@@ -456,11 +539,12 @@ void csysi_read_proc(const MIDIPacketList * pktlist,
   struct iovec vector[2];
   MIDIPacket * p;
   int num, idx, retry;
+  unsigned char ext;
 
   /* initialize preamble */
 
   preamble[CSYSI_PREAMBLE_POS_MARKER] = CSYSI_PREAMBLE_FLAGS_MARKER;
-  preamble[CSYSI_PREAMBLE_POS_CONTROL] = 
+  preamble[CSYSI_PREAMBLE_POS_CONTROL] = ext = 
     CSYSI_PREAMBLE_FLAGS_EXT & ((unsigned char) *((int *) srcConnRefCon));
 
   /* initialize write vector -- update for timestamp support */
@@ -479,7 +563,7 @@ void csysi_read_proc(const MIDIPacketList * pktlist,
       retry = idx = 0;
 
       do {
-	if ((vector[1].iov_len = csysi_packet_shorten(p, mdata, &idx)))
+	if ((vector[1].iov_len = csysi_packet_shorten(p, mdata, &idx, ext)))
 	  while (writev(csysi_readproc_pipepair[1], vector, 2) < 0)
 	    if ((errno != EINTR) || (++retry >= CSYSI_RETRY_MAX))
 	      {
@@ -513,12 +597,18 @@ void csysi_notify_proc(const MIDINotification * message,
 /****************************************************************/
 
 int csysi_packet_shorten(MIDIPacket * p, unsigned char * mdata,
-			 int * idx)
+			 int * idx, unsigned char ext)
 {
   int len = 0;
 
   do {
-    switch (p->data[*idx] & 0xF0) {
+
+    if (CSYSI_SYSTEM_GATE == CSYSI_PASS_SYSTEM_COMMANDS)
+      if (csysi_ports[ext].sysex_size && (p->data[*idx] & 0xF0u))
+	if (csysi_packet_arblength(p, mdata, idx, &len, ext) || (*idx == p->length))
+	  return len;
+
+    switch (p->data[*idx] & 0xF0u) {
     case CSYS_MIDI_NOTEOFF:
     case CSYS_MIDI_NOTEON:
     case CSYS_MIDI_PTOUCH:
@@ -534,11 +624,7 @@ int csysi_packet_shorten(MIDIPacket * p, unsigned char * mdata,
 	    return len;
 	}
       else
-	{
-	  mdata[len] = CSYS_MIDI_SYSTEM_RESET; /* CoreMIDI bug recovery */
-	  *idx = p->length;
-	  return len + 1;   
-	}
+	return csysi_bugrecovery(p, mdata, idx, len);
       continue;
     case CSYS_MIDI_PROGRAM:
     case CSYS_MIDI_CTOUCH:
@@ -551,47 +637,139 @@ int csysi_packet_shorten(MIDIPacket * p, unsigned char * mdata,
 	    return len;     
 	}
       else
-	{
-	  mdata[len] = CSYS_MIDI_SYSTEM_RESET; /* CoreMIDI bug recovery */
-	  *idx = p->length;
-	  return len + 1;   
-	}
+	return csysi_bugrecovery(p, mdata, idx, len);
       continue;
     case CSYS_MIDI_SYSTEM:
       switch (p->data[*idx]) {
-      case CSYS_MIDI_SYSTEM_RESET:        
-	memcpy(&(mdata[len]), &(p->data[*idx]), 1);
-	if (p->length == (*idx += 1))
-	  return len + 1;
-	if ((len += 1) > (CSYSI_SENDSIZE - 3))
-	  return len;     
-	continue;
-      case CSYS_MIDI_SYSTEM_SONG_PP:                  /* skip 3-octet commands  */ 
-	if (p->length == (*idx += 3))
-	  return len;
-	continue;
-      case CSYS_MIDI_SYSTEM_QFRAME:                   /* skip 2-octet commands */ 
-      case CSYS_MIDI_SYSTEM_SONG_SELECT: 
-	if (p->length == (*idx += 2))
-	  return len;
-	continue;
-      case CSYS_MIDI_SYSTEM_TUNE_REQUEST:             /* skip 1-octet commands */ 
-      case CSYS_MIDI_SYSTEM_CLOCK:     
-      case CSYS_MIDI_SYSTEM_TICK:
+      case CSYS_MIDI_SYSTEM_TICK:                                /* 1 octet */ 
+      case CSYS_MIDI_SYSTEM_TUNE_REQUEST:                   
+      case CSYS_MIDI_SYSTEM_CLOCK:
       case CSYS_MIDI_SYSTEM_START:
       case CSYS_MIDI_SYSTEM_CONTINUE:
       case CSYS_MIDI_SYSTEM_STOP:
       case CSYS_MIDI_SYSTEM_UNUSED3:
       case CSYS_MIDI_SYSTEM_SENSE:
-      case CSYS_MIDI_SYSTEM_SYSEX_END:
+      case CSYS_MIDI_SYSTEM_RESET:
+	if (CSYSI_SYSTEM_GATE == CSYSI_BLOCK_SYSTEM_COMMANDS)
+	  {
+	    if (p->length == (*idx += 1))
+	      return len;
+	  }
+	else
+	  {
+	    memcpy(&(mdata[len]), &(p->data[*idx]), 1);
+	    if (p->length == (*idx += 1))
+	      return len + 1;
+	    if ((len += 1) > (CSYSI_SENDSIZE - 3))
+	      return len;
+	  }
+	continue;
+      case CSYS_MIDI_SYSTEM_SYSEX_END:                      
 	if (p->length == (*idx += 1))
 	  return len;
 	continue;
-      case CSYS_MIDI_SYSTEM_SYSEX_START:          /* arbitrary-length commands */
-      case CSYS_MIDI_SYSTEM_UNUSED1:   
+      case CSYS_MIDI_SYSTEM_QFRAME:                              /* 2 octets */ 
+      case CSYS_MIDI_SYSTEM_SONG_SELECT:
+	if (CSYSI_SYSTEM_GATE == CSYSI_BLOCK_SYSTEM_COMMANDS)
+	  {
+	    if (p->length == (*idx += 2))
+	      return len;	 
+	  }
+	else
+	  {
+	    if ((p->length >= (*idx + 2)) && (p->data[*idx + 1] < 128))
+	      {
+		memcpy(&(mdata[len]), &(p->data[*idx]), 2);
+		if (p->length == (*idx += 2))
+		  return len + 2;
+		if ((len += 2) > (CSYSI_SENDSIZE - 3))
+		  return len;     
+	      }
+	    else
+	      return csysi_bugrecovery(p, mdata, idx, len);
+	  }
+      continue;
+      case CSYS_MIDI_SYSTEM_SONG_PP:                             /* 3 octets */ 
+	if (CSYSI_SYSTEM_GATE == CSYSI_BLOCK_SYSTEM_COMMANDS)
+	  {
+	    if (p->length == (*idx += 3))
+	      return len;
+	  }
+	else
+	  {
+	    if ((p->length >= (*idx + 3)) && (p->data[*idx + 1] < 128)
+		&& (p->data[*idx + 2] < 128))
+	      {
+		memcpy(&(mdata[len]), &(p->data[*idx]), 3);
+		if (p->length == (*idx += 3))
+		  return len + 3;
+		if ((len += 3) > (CSYSI_SENDSIZE - 3))
+		  return len;
+	      }
+	    else
+	      return csysi_bugrecovery(p, mdata, idx, len);
+	  }
+	continue;
+      case CSYS_MIDI_SYSTEM_UNUSED1:           /* undefined Common commands */
       case CSYS_MIDI_SYSTEM_UNUSED2:
+	if (CSYSI_SYSTEM_GATE == CSYSI_PASS_SYSTEM_COMMANDS)
+	  {
+	    /* case 1: zero data octets */
+	    
+	    if ((((*idx) + 1) == p->length) || (p->data[(*idx) + 1] & 0x80u))
+	      {
+		memcpy(&(mdata[len]), &(p->data[*idx]), 1);
+		if (p->length == (*idx += 1))
+		  return len + 1;
+		if ((len += 1) > (CSYSI_SENDSIZE - 3))
+		  return len;
+		continue;
+	      }
+
+	    /* case 2: one data octet */
+	    
+	    if ((((*idx) + 2) == p->length) || (p->data[(*idx) + 2] & 0x80u))
+	      {
+		memcpy(&(mdata[len]), &(p->data[*idx]), 2);
+		if (p->length == (*idx += 2))
+		  return len + 2;
+		if ((len += 2) > (CSYSI_SENDSIZE - 3))
+		  return len;
+		continue;
+	      }
+
+	    /* case 3: two data octets */
+	    
+	    if ((((*idx) + 3) == p->length) || (p->data[(*idx) + 3] & 0x80u))
+	      {
+		memcpy(&(mdata[len]), &(p->data[*idx]), 3);
+		if (p->length == (*idx += 3))
+		  return len + 3;
+		if ((len += 3) > (CSYSI_SENDSIZE - 3))
+		  return len;
+		continue;
+	      }
+	  }
 	*idx += 1;
-	while ((*idx < p->length) && !(p->data[(*idx)++] & 0x80)); /* skip data */
+	while ((*idx < p->length) && !(p->data[(*idx)] & 0x80u))
+	  (*idx)++;
+	if (*idx == p->length)
+	  return len;
+	continue;
+      case CSYS_MIDI_SYSTEM_SYSEX_START:                 /* SysEx commands */
+	if (CSYSI_SYSTEM_GATE == CSYSI_BLOCK_SYSTEM_COMMANDS)
+	  {
+	    if (p->data[(*idx)] == CSYS_MIDI_SYSTEM_SYSEX_START)
+	      *idx += 1;
+	    while ((*idx < p->length) && !(p->data[(*idx)] & 0x80u))
+	      (*idx)++;
+	    if (p->data[(*idx)] == CSYS_MIDI_SYSTEM_SYSEX_END)
+	      (*idx)++;
+	  }
+	else
+	  if (csysi_packet_arblength(p, mdata, idx, &len, ext))
+	    return len;
+
 	if (*idx == p->length)
 	  return len;
 	continue;
@@ -599,7 +777,19 @@ int csysi_packet_shorten(MIDIPacket * p, unsigned char * mdata,
       *idx = p->length;                                /* should never run     */
       return len;
     default:                                           /* a sysex continuation */
-      while ((*idx < p->length) && !(p->data[(*idx)++] & 0x80));   
+      if (CSYSI_SYSTEM_GATE == CSYSI_BLOCK_SYSTEM_COMMANDS)
+	{
+	  while ((*idx < p->length) && !(p->data[(*idx)] & 0x80u))
+	    (*idx)++;
+	  if (p->data[(*idx)] == CSYS_MIDI_SYSTEM_SYSEX_END)
+	    (*idx)++;
+	  if (*idx == p->length)
+	    return len;
+	}
+      else
+	if (csysi_packet_arblength(p, mdata, idx, &len, ext))
+	  return len;
+
       if (*idx == p->length)
 	return len;
       continue;
@@ -611,6 +801,132 @@ int csysi_packet_shorten(MIDIPacket * p, unsigned char * mdata,
   return len;
 } 
 
+/****************************************************************/
+/*             recover from CoreMIDI legacy bugs                */
+/****************************************************************/
+
+int csysi_bugrecovery(MIDIPacket * p, unsigned char * mdata,
+		   int * idx, int len)
+{
+  mdata[len] = CSYS_MIDI_SYSTEM_RESET;
+  *idx = p->length;
+  return len + 1;   
+}
+
+/****************************************************************/
+/*          parse arbitrary-length command bodies               */
+/****************************************************************/
+
+int csysi_packet_arblength(MIDIPacket * p, unsigned char * mdata,
+			   int * idx, int * len, unsigned char ext)
+{
+
+  if (p->data[(*idx)] == CSYS_MIDI_SYSTEM_SYSEX_START)
+    {
+      csysi_ports[ext].sysex_size = 0;
+      *idx += 1;
+    }
+
+  while (*idx < p->length)
+    if (p->data[(*idx)] & 0x80u)
+      {
+	if (p->data[(*idx)] == CSYS_MIDI_SYSTEM_SYSEX_END)
+	  (*idx)++;
+	
+	switch (csysi_ports[ext].sysex_size) {
+	case 2:
+	  if (csysi_ports[ext].sysbuff[0] == 0x7Du)
+	    {
+	      if (csysi_ports[ext].sysbuff[1] & CSYSI_UNDEFINED_F4)
+		mdata[(*len)++] = CSYS_MIDI_SYSTEM_UNUSED1;
+	      else
+		mdata[(*len)++] = CSYS_MIDI_SYSTEM_UNUSED2;
+	      if ((p->length == (*idx)) || ((*len) > (CSYSI_SENDSIZE - 3)))
+		return CSYSI_COMMAND_COMPLETED;
+	    }
+	  break;
+	case 3:
+	  if (csysi_ports[ext].sysbuff[0] == 0x7Du)
+	    {
+	      if (csysi_ports[ext].sysbuff[1] & CSYSI_UNDEFINED_F4)
+		mdata[(*len)++] = CSYS_MIDI_SYSTEM_UNUSED1;
+	      else
+		mdata[(*len)++] = CSYS_MIDI_SYSTEM_UNUSED2;
+	      mdata[(*len)++] = csysi_ports[ext].sysbuff[2];
+	      if ((p->length == (*idx)) || ((*len) > (CSYSI_SENDSIZE - 3)))
+		return CSYSI_COMMAND_COMPLETED;
+	    }
+	  break;
+	case 4:
+	  if (csysi_ports[ext].sysbuff[0] == 0x7Du)
+	    {
+	      if (csysi_ports[ext].sysbuff[1] & CSYSI_UNDEFINED_F4)
+		mdata[(*len)++] = CSYS_MIDI_SYSTEM_UNUSED1;
+	      else
+		mdata[(*len)++] = CSYS_MIDI_SYSTEM_UNUSED2;
+	      mdata[(*len)++] = csysi_ports[ext].sysbuff[2];
+	      mdata[(*len)++] = csysi_ports[ext].sysbuff[3];
+	      if ((p->length == (*idx)) || ((*len) > (CSYSI_SENDSIZE - 3)))
+		return CSYSI_COMMAND_COMPLETED;
+	    }
+	  if ((csysi_ports[ext].sysbuff[0] == 0x7Eu) &&
+	      (csysi_ports[ext].sysbuff[1] == 0x7Fu) &&
+	      (csysi_ports[ext].sysbuff[2] == 0x09u) &&
+	      ((csysi_ports[ext].sysbuff[3] == 0x01u) ||
+	       (csysi_ports[ext].sysbuff[3] == 0x02u)))
+	    {
+	      mdata[(*len)++] = CSYS_MIDI_GMRESET;
+	      mdata[(*len)++] = csysi_ports[ext].sysbuff[3];
+	      if ((p->length == (*idx)) || ((*len) > (CSYSI_SENDSIZE - 3)))
+		return CSYSI_COMMAND_COMPLETED;
+	    }
+	  break;
+	case 6:   
+	  if ((csysi_ports[ext].sysbuff[0] == 0x7Fu) &&
+	      (csysi_ports[ext].sysbuff[1] == 0x7Fu) &&
+	      (csysi_ports[ext].sysbuff[2] == 0x04u) &&
+	      (csysi_ports[ext].sysbuff[3] == 0x01u) &&
+	      (csysi_ports[ext].sysbuff[4] < 0x80u)  &&
+	      (csysi_ports[ext].sysbuff[5] < 0x80u))
+	    {
+	      mdata[(*len)++] = CSYS_MIDI_MVOLUME;
+	      mdata[(*len)++] = csysi_ports[ext].sysbuff[4];
+	      mdata[(*len)++] = csysi_ports[ext].sysbuff[5];
+	      if ((p->length == (*idx)) || ((*len) > (CSYSI_SENDSIZE - 3)))
+		return CSYSI_COMMAND_COMPLETED;
+	    }
+	  break;
+	case 8:   
+	  if ((csysi_ports[ext].sysbuff[0] == 0x43u) &&
+	      (csysi_ports[ext].sysbuff[1] == 0x73u) &&
+	      (csysi_ports[ext].sysbuff[2] == 0x7Fu) &&
+	      (csysi_ports[ext].sysbuff[3] == 0x32u) &&
+	      (csysi_ports[ext].sysbuff[4] == 0x11u) &&
+	      (csysi_ports[ext].sysbuff[5] < 0x80u) &&
+	      (csysi_ports[ext].sysbuff[6] < 0x80u) &&
+	      (csysi_ports[ext].sysbuff[7] < 0x80u))
+	    {
+	      mdata[(*len)++] = CSYS_MIDI_MANUEX;
+	      mdata[(*len)++] = csysi_ports[ext].sysbuff[6];
+	      mdata[(*len)++] = csysi_ports[ext].sysbuff[7];
+	      if ((p->length == (*idx)) || ((*len) > (CSYSI_SENDSIZE - 3)))
+		return CSYSI_COMMAND_COMPLETED;
+	    }
+	  break; 
+	}
+	csysi_ports[ext].sysex_size = 0;
+	break;                               /* while */
+      }
+    else
+      {
+	if (csysi_ports[ext].sysex_size < CSYSI_SYSEXSIZE)
+	  csysi_ports[ext].sysbuff[csysi_ports[ext].sysex_size++] =
+	    p->data[*idx];
+	(*idx)++;
+      }
+
+  return CSYSI_COMMAND_ONGOING;
+}
 
 /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
 /*                 low-level functions                          */
