@@ -2,20 +2,36 @@
 /*
 #    Sfront, a SAOL to C translator    
 #    This file: Network library -- global variables
-#    Copyright (C) 1999  Regents of the University of California
 #
-#    This program is free software; you can redistribute it and/or modify
-#    it under the terms of the GNU General Public License (Version 2) as
-#    published by the Free Software Foundation.
+# Copyright (c) 2000, Regents of the University of California
+# All rights reserved.
 #
-#    This program is distributed in the hope that it will be useful,
-#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#    GNU General Public License for more details.
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions are
+# met:
 #
-#    You should have received a copy of the GNU General Public License
-#    along with this program; if not, write to the Free Software
-#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+#  Redistributions of source code must retain the above copyright
+#  notice, this list of conditions and the following disclaimer.
+#
+#  Redistributions in binary form must reproduce the above copyright
+#  notice, this list of conditions and the following disclaimer in the
+#  documentation and/or other materials provided with the distribution.
+#
+#  Neither the name of the University of California, Berkeley nor the
+#  names of its contributors may be used to endorse or promote products
+#  derived from this software without specific prior written permission.
+#
+# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #
 #    Maintainer: John Lazzaro, lazzaro@cs.berkeley.edu
 */
@@ -80,14 +96,14 @@ unsigned char nsys_netout_markbit; /* for setting markerbit */
 
 struct nsys_payinfo nsys_payload_types[NSYS_RTP_PAYSIZE] = 
 { 
-  {0, 97, "X-MP4SA-1", (int)(ARATE + 0.5F)},
-  {1, 96, "X-MP4SA-0", (int)(ARATE + 0.5F)}
+  {NSYS_MPEG4_PINDEX,  96, "mpeg4-generic", (int)(ARATE + 0.5F)},
+  {NSYS_NATIVE_PINDEX, 97, "rtp-midi", (int)(ARATE + 0.5F)}
 };
 #endif
 
 struct nsys_payinfo nsys_payload_types[NSYS_RTP_PAYSIZE] = 
 { 
-  {0, 96, "X-MP4SA-0", (int)(ARATE + 0.5F)}
+  {NSYS_MPEG4_PINDEX, 96, "pre1-mpeg4-generic", (int)(ARATE + 0.5F)}
 };
 
 /**************/
@@ -145,9 +161,15 @@ unsigned char nsys_buff[NSYS_BUFFSIZE];
 long nsys_bufflen;
 long nsys_buffcnt;
 
-/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
-/* mpeg4-samidi: recovery journal sender state */
-/*_____________________________________________*/
+/*~~~~~~~~~~~*/
+/* rtp-midi  */
+/*___________*/
+
+/*************************/
+/* command section flags */
+/*************************/
+
+unsigned char nsys_netout_sm_header;
 
 /*************************/
 /* checkpoint management */
@@ -193,12 +215,34 @@ unsigned char nsys_netout_jsend_channel_size;
 
 nsys_netout_jsend_state nsys_netout_jsend[CSYS_MIDI_NUMCHAN];
 
+/***********************/
+/* sender system state */
+/***********************/
+
+nsys_netout_jsend_system_state nsys_netout_jsend_system;
+nsys_netout_jsend_xstack_element nsys_netout_jsend_xpile[NSYS_SM_CX_MAXSLOTS]; 
+nsys_netout_jsend_xstack_element * nsys_netout_jsend_xstackfree; 
+
 /**************************/
 /* receiver channel state */
 /**************************/
 
 nsys_netout_jrecv_state * nsys_recvfree;  
-  
+nsys_netout_jrecv_system_state * nsys_recvsysfree;
+
+/****************************/
+/* supported SysEx commands */
+/****************************/
+
+unsigned char nsys_netout_sysconst_manuex[CSYS_MIDI_MANUEX_SIZE] = {
+  0xF0, 0x43, 0x73, 0x7F, 0x32, 0x11, 0x00, 0x00, 0x00, 0xF7 };
+
+unsigned char nsys_netout_sysconst_mvolume[CSYS_MIDI_MVOLUME_SIZE] = {
+  0xF0, 0x7F, 0x7F, 0x04, 0x01, 0x00, 0x00, 0xF7};
+
+unsigned char nsys_netout_sysconst_gmreset[CSYS_MIDI_GMRESET_SIZE] = {
+  0xF0, 0x7E, 0x7F, 0x09, 0x00, 0xF7};
+
 
 /*******/
 /* SIP */
