@@ -2,20 +2,36 @@
 /*
 #    Sfront, a SAOL to C translator    
 #    This file: Network library -- RTP functions
-#    Copyright (C) 1999  Regents of the University of California
 #
-#    This program is free software; you can redistribute it and/or modify
-#    it under the terms of the GNU General Public License (Version 2) as
-#    published by the Free Software Foundation.
+# Copyright (c) 2000, Regents of the University of California
+# All rights reserved.
 #
-#    This program is distributed in the hope that it will be useful,
-#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#    GNU General Public License for more details.
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions are
+# met:
 #
-#    You should have received a copy of the GNU General Public License
-#    along with this program; if not, write to the Free Software
-#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+#  Redistributions of source code must retain the above copyright
+#  notice, this list of conditions and the following disclaimer.
+#
+#  Redistributions in binary form must reproduce the above copyright
+#  notice, this list of conditions and the following disclaimer in the
+#  documentation and/or other materials provided with the distribution.
+#
+#  Neither the name of the University of California, Berkeley nor the
+#  names of its contributors may be used to endorse or promote products
+#  derived from this software without specific prior written permission.
+#
+# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #
 #    Maintainer: John Lazzaro, lazzaro@cs.berkeley.edu
 */
@@ -367,8 +383,8 @@ int nsys_netin_clear_mset(unsigned char * buff, long * fill, long size)
   s.mset = nsys_powerup_mset;
   nsys_powerup_mset = 0; 
  
-  return nsys_netin_journal_addcmd(&s, buff, fill, size, 
-				   CSYS_MIDI_POWERUP, 0, 0);
+  return nsys_netin_journal_addcmd_three(&s, buff, fill, size, 
+					 CSYS_MIDI_POWERUP, 0, 0);
 }
 
 /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
@@ -474,7 +490,7 @@ int nsys_netout_excheck(void)
 
   nsys_netout_rtcp_initpackets();
 
-  nsys_sent_last = nsys_sent_last = 0;
+  nsys_sent_this = nsys_sent_last = 0;
   nsys_sent_octets = nsys_sent_packets = 0;
 
   if ((sptr = nsys_srcroot))
@@ -1195,6 +1211,12 @@ void nsys_delete_ssrc(unsigned long ssrc, char * reason)
 	sptr->jrecv[i]->next = nsys_recvfree;
 	nsys_recvfree = sptr->jrecv[i];
       }
+
+  if (sptr->jrecvsys)
+    {
+      sptr->jrecvsys->next = nsys_recvsysfree;
+      nsys_recvsysfree = sptr->jrecvsys;
+    }
   
 #if (NSYS_LATENOTES_DEBUG == NSYS_LATENOTES_DEBUG_ON)
   printf("mset%i packets expected: %i\n", sptr->mset,
