@@ -2,20 +2,36 @@
 /*
 #    Sfront, a SAOL to C translator    
 #    This file: Network library -- RTP functions
-#    Copyright (C) 1999  Regents of the University of California
 #
-#    This program is free software; you can redistribute it and/or modify
-#    it under the terms of the GNU General Public License (Version 2) as
-#    published by the Free Software Foundation.
+# Copyright (c) 2000, Regents of the University of California
+# All rights reserved.
 #
-#    This program is distributed in the hope that it will be useful,
-#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#    GNU General Public License for more details.
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions are
+# met:
 #
-#    You should have received a copy of the GNU General Public License
-#    along with this program; if not, write to the Free Software
-#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+#  Redistributions of source code must retain the above copyright
+#  notice, this list of conditions and the following disclaimer.
+#
+#  Redistributions in binary form must reproduce the above copyright
+#  notice, this list of conditions and the following disclaimer in the
+#  documentation and/or other materials provided with the distribution.
+#
+#  Neither the name of the University of California, Berkeley nor the
+#  names of its contributors may be used to endorse or promote products
+#  derived from this software without specific prior written permission.
+#
+# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #
 #    Maintainer: John Lazzaro, lazzaro@cs.berkeley.edu
 */
@@ -39,6 +55,7 @@ int nsys_setup(int block)
   struct hostent * myinfo;
   struct nsys_source * newsrc;
   struct nsys_netout_jrecv_state * newrecv;
+  struct nsys_netout_jrecv_system_state * newrecvsys;
   struct sockaddr_in iclient_addr; 
   struct timeval tv;
   unsigned char session_digest[NSYS_MD5_LENGTH];
@@ -231,6 +248,19 @@ int nsys_setup(int block)
       nsys_recvfree = newrecv;
     }
 
+  for (i = 0; i < NSYS_MSETS; i++)
+    {
+      newrecvsys = calloc(1, sizeof(struct nsys_netout_jrecv_system_state));
+      newrecvsys->next = nsys_recvsysfree;
+      nsys_recvsysfree = newrecvsys;
+    }
+
+  for (i = 0; i < NSYS_SM_CX_MAXSLOTS; i++)
+    {
+      nsys_netout_jsend_xpile[i].next = nsys_netout_jsend_xstackfree; 
+      nsys_netout_jsend_xstackfree = &(nsys_netout_jsend_xpile[i]); 
+    }
+
   /************************/
   /* setup network output */
   /************************/
@@ -246,12 +276,14 @@ int nsys_setup(int block)
 
   nsys_netout_rtp_packet[NSYS_RTPLOC_BYTE1] = NSYS_RTPVAL_BYTE1;
 
-  if (nsys_feclevel == NSYS_SM_FEC_NONE)
-    nsys_netout_markbit = NSYS_RTPVAL_SETMARK;
+  nsys_netout_markbit = NSYS_RTPVAL_SETMARK;   /* required by mpeg4-generic */
 
   memcpy(&(nsys_netout_rtp_packet[NSYS_RTPLOC_SSRC]), &nsys_myssrc_net,
 	 sizeof(long));
 
+  if (nsys_feclevel != NSYS_SM_FEC_NONE)
+    nsys_netout_sm_header = NSYS_SM_SETJ;
+
   /*********************/
   /* setup RTCP system */
   /*********************/
@@ -262,12 +294,12 @@ int nsys_setup(int block)
   nsys_rtcp_ex  = NSYS_RTCPEX_NULLROOT;
   nsys_rtcp_ex |= (NSYS_RTCPEX_RTCPSIP | NSYS_RTCPEX_RTPSIP); 
 
-  /*******************************/
-  /* setup sa-midi packetization */
-  /*******************************/
+  /********************************/
+  /* setup rtp-midi packetization */
+  /********************************/
 
   nsys_netout_jsend_checkpoint_seqnum = nsys_netout_seqnum;
-  nsys_netout_jsend_header[NSYS_SM_JH_LOC_FLAGS] = NSYS_SM_JH_SETA;
+  nsys_netout_jsend_header[NSYS_SM_JH_LOC_FLAGS] = 0;
 
   for (i = 0; i < CSYS_MIDI_NUMCHAN; i++)
     nsys_netout_jsend[i].chan = i;
@@ -444,6 +476,10 @@ int nsys_netin_rtpstats(nsys_source * sptr, unsigned char * packet)
   int seqinc, delta;
   int rtpcode;
 
+#if (NSYS_LATENOTES_DEBUG == NSYS_LATENOTES_DEBUG_ON)
+  int mlen, first;
+#endif
+
   seq = ntohs(*((unsigned short *)&(packet[NSYS_RTPLOC_SEQNUM])));
 
   delta = nsys_netout_tstamp - 
@@ -543,35 +579,55 @@ int nsys_netin_rtpstats(nsys_source * sptr, unsigned char * packet)
 	      (nsys_netout_tstamp - sptr->tm_first)/ARATE,
 	      ((int)(delta + sptr->tm_convert))/ARATE);
 
-      if ((packet[NSYS_RTPLEN_HDR] & NSYS_SM_MLENMAX) == 0)
+      /* not checking length, but not dangerous */
+
+      mlen = NSYS_SM_MLENMASK & packet[NSYS_RTPLEN_HDR];
+      first = NSYS_RTPLEN_HDR + 1;
+
+      if (packet[NSYS_RTPLEN_HDR] & NSYS_SM_CHKB)
+	{
+	  mlen = packet[NSYS_RTPLEN_HDR+1] + (mlen << 8);
+	  first++;
+	}
+
+      if (mlen == 0)
 	fprintf(sptr->tm_fd, "Empty\n");
       else
-	switch(packet[NSYS_RTPLEN_HDR+1] & 0xF0) {
-	case CSYS_MIDI_NOTEOFF:
-	  fprintf(sptr->tm_fd, "NoteOff\n");
-	  break;
-	case CSYS_MIDI_NOTEON:
-	  if (packet[NSYS_RTPLEN_HDR+3])
-	    fprintf(sptr->tm_fd, "NoteOn\n");
-	  else
+	if (packet[NSYS_RTPLEN_HDR] & NSYS_SM_CHKZ)
+	  fprintf(sptr->tm_fd, "Delta-Time\n");
+	else
+	  switch(packet[first] & 0xF0) {
+	  case CSYS_MIDI_NOTEOFF:
 	    fprintf(sptr->tm_fd, "NoteOff\n");
-	  break;
-	case CSYS_MIDI_PTOUCH:
-	  fprintf(sptr->tm_fd, "PTouch\n");
-	  break;
-	case CSYS_MIDI_CC:
-	  fprintf(sptr->tm_fd, "CChange\n");
-	  break;
-	case CSYS_MIDI_PROGRAM:
-	  fprintf(sptr->tm_fd, "PChange\n");
-	  break;
-	case CSYS_MIDI_CTOUCH:
-	  fprintf(sptr->tm_fd, "CTouch\n");
-	  break;
-	case CSYS_MIDI_WHEEL:
-	  fprintf(sptr->tm_fd, "PWheel\n");
-	  break;
-	}
+	    break;
+	  case CSYS_MIDI_NOTEON:
+	    if (packet[first + 2])
+	      fprintf(sptr->tm_fd, "NoteOn\n");
+	    else
+	      fprintf(sptr->tm_fd, "NoteOff\n");
+	    break;
+	  case CSYS_MIDI_PTOUCH:
+	    fprintf(sptr->tm_fd, "PTouch\n");
+	    break;
+	  case CSYS_MIDI_CC:
+	    fprintf(sptr->tm_fd, "CChange\n");
+	    break;
+	  case CSYS_MIDI_PROGRAM:
+	    fprintf(sptr->tm_fd, "PChange\n");
+	    break;
+	  case CSYS_MIDI_CTOUCH:
+	    fprintf(sptr->tm_fd, "CTouch\n");
+	    break;
+	  case CSYS_MIDI_WHEEL:
+	    fprintf(sptr->tm_fd, "PWheel\n");
+	    break;
+	  case CSYS_MIDI_SYSTEM:
+	    fprintf(sptr->tm_fd, "System\n");
+	    break;
+	  default:
+	    fprintf(sptr->tm_fd, "Data Octet\n");
+	    break;
+	  }
     }
 
 #endif
@@ -691,7 +747,7 @@ void nsys_netin_ptypeset(struct nsys_source * sptr, unsigned char ptype)
 }
 
 /****************************************************************/
-/*              return a new jrecv pointer                      */
+/*              return a new jrecv channel pointer              */
 /****************************************************************/
 
 nsys_netout_jrecv_state * nsys_netin_newrecv(unsigned char chan)
@@ -711,6 +767,26 @@ nsys_netout_jrecv_state * nsys_netin_newrecv(unsigned char chan)
   ret->chan = chan;
   return ret;
 }
+/***************************************************************/
+/*              return a new jrecv system pointer              */
+/***************************************************************/
+
+nsys_netout_jrecv_system_state * nsys_netin_newrecvsys(void)
+
+{
+  nsys_netout_jrecv_system_state * ret;
+
+  if (nsys_recvfree)
+    {
+      ret = nsys_recvsysfree;
+      nsys_recvsysfree = nsys_recvsysfree->next;
+      memset(ret, 0, sizeof(struct nsys_netout_jrecv_system_state));
+    }
+  else
+    ret = calloc(1, sizeof(struct nsys_netout_jrecv_system_state));
+
+  return ret;
+}
 
 /****************************************************************/
 /*       determines client IP number, returns NULL on error     */
@@ -738,8 +814,9 @@ char * nsys_find_clientip(char * ip)
       return NULL;
     }
 
-  /* look through addresses, return a useful one */
-  /* thanks to Jens-Uwe Mager for BSD 4.4 updates */
+  /* look through addresses, return a useful one    */
+  /* thanks to Jens-Uwe Mager for BSD 4.4 updates   */
+  /* found his code snippet on comp.unix.programmer */
 
   ifr = ifc.ifc_req;
 
