@@ -2,20 +2,36 @@
 /*
 #    Sfront, a SAOL to C translator    
 #    This file: Network library -- SIP functions
-#    Copyright (C) 1999  Regents of the University of California
 #
-#    This program is free software; you can redistribute it and/or modify
-#    it under the terms of the GNU General Public License (Version 2) as
-#    published by the Free Software Foundation.
+# Copyright (c) 2000, Regents of the University of California
+# All rights reserved.
 #
-#    This program is distributed in the hope that it will be useful,
-#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#    GNU General Public License for more details.
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions are
+# met:
 #
-#    You should have received a copy of the GNU General Public License
-#    along with this program; if not, write to the Free Software
-#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+#  Redistributions of source code must retain the above copyright
+#  notice, this list of conditions and the following disclaimer.
+#
+#  Redistributions in binary form must reproduce the above copyright
+#  notice, this list of conditions and the following disclaimer in the
+#  documentation and/or other materials provided with the distribution.
+#
+#  Neither the name of the University of California, Berkeley nor the
+#  names of its contributors may be used to endorse or promote products
+#  derived from this software without specific prior written permission.
+#
+# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #
 #    Maintainer: John Lazzaro, lazzaro@cs.berkeley.edu
 */
@@ -272,7 +288,7 @@ void nsys_netin_invite(int fd, struct sockaddr_in * addr,
   unsigned short port, nat_port, ptype;
   int state, len;
   int has_nat, has_session, has_time, has_ssrc; 
-  int  has_dnsname, has_port, has_ip, has_map;
+  int has_dnsname, has_port, has_ip, has_map;
 
   /*****************************************************************/
   /* do weak and strong security checks, leave function if failure */
@@ -402,7 +418,7 @@ void nsys_netin_invite(int fd, struct sockaddr_in * addr,
 	sptr = sptr->xtra;
       
       if ((sptr == NULL) && nsys_netin_noreplay(ip, port, sdp_time) &&
-	  nsys_netin_payvalid(marray, mlen, fd) && 
+	  nsys_netin_payvalid(marray, mlen, fd) &&
 	  (sptr = nsys_netin_addsrc(fd, ssrc, ip, port)))
 	{
 	  
@@ -690,20 +706,70 @@ void nsys_createsip(int fd, char * method, unsigned char * sip,
   if (len > NSYS_UDPMAXSIZE)
     NSYS_ERROR_TERMINATE("SIP INVITE packet creation error [3]");
 
-  /*******************/
-  /* append RTP maps */
-  /*******************/
-
   for (i = 0; i < NSYS_RTP_PAYSIZE; i++)
     {
+      /**********************/
+      /* append rtpmap line */
+      /**********************/
+
       len += snprintf(&(sip[len]), NSYS_UDPMAXSIZE+1 - len, 
-		      "a=rtpmap:%hhu %s/%i\n", 
+		      "a=rtpmap: %hhu %s/%i\n", 
 		      nsys_payload_types[i].ptype, 
 		      nsys_payload_types[i].name, 
 		      nsys_payload_types[i].srate);
 
       if ((len > NSYS_UDPMAXSIZE))
 	NSYS_ERROR_TERMINATE("SIP INVITE packet creation error [4]");
+
+      /*********************/
+      /* append fmtp lines */
+      /*********************/
+
+      switch (nsys_payload_types[i].pindex) {
+      case NSYS_MPEG4_PINDEX:
+	len += snprintf(&(sip[len]), NSYS_UDPMAXSIZE+1 - len,
+
+	"a=fmtp: %hhu streamtype=5; mode=rtp-midi; profile-level-id=12;\n"
+	"a=fmtp: %hhu config=\"\"; render=synthetic; rinit=audio/asc; \n"
+
+	"a=fmtp: %hhu tsmode=buffer; octpos=last; mperiod=%lu;\n"
+	"a=fmtp: %hhu ptime=0; maxptime=0; ch_unused=DEFMQVX;%s\n"
+
+	"a=fmtp: %hhu guardtime=%lu;\n",
+
+			/* streamtype line */
+
+			nsys_payload_types[i].ptype,
+
+			/* config line */
+
+			nsys_payload_types[i].ptype,
+
+			/* tsmode line */
+
+			nsys_payload_types[i].ptype,
+			(unsigned long) ACYCLE,
+
+			/* ptime line */
+
+			nsys_payload_types[i].ptype,
+			(nsys_feclevel != NSYS_SM_FEC_NONE) 
+			? "" : " j_sec=none;",
+
+			/* guardtime line */
+
+			nsys_payload_types[i].ptype,
+			(unsigned long)(NSYS_SM_GUARD_MAXTIME*ARATE + 0.5F)
+
+			);
+
+	if ((len > NSYS_UDPMAXSIZE))
+	  NSYS_ERROR_TERMINATE("SIP INVITE packet creation error [5]");
+	break;
+      default:
+	break;
+      }
+
     }
 
   /***************************************/
@@ -1174,18 +1240,18 @@ int nsys_netin_make_marray(struct nsys_payinfo ** marray, char * media)
 
 
 /****************************************************************/
-/*                 sets a media payloads                       */
+/*                 sets media payloads                          */
 /****************************************************************/
 
 int nsys_netin_set_marray(char * line, struct nsys_payinfo marray[], int mlen)
 
 {
   unsigned short ptype;  /* sscanf needs a short, not a char */
-  char name[16];
+  char name[32];
   int srate; 
   int i, num;
 
-  num = sscanf(line, "a=rtpmap:%hu %15[^/]/%i", &ptype, name, &srate);
+  num = sscanf(line, "a=rtpmap:%hu %31[^/]/%i", &ptype, name, &srate);
 
   if ((num != 2) && (num != 3))
     return 0;
@@ -1216,7 +1282,8 @@ int nsys_netin_set_marray(char * line, struct nsys_payinfo marray[], int mlen)
 /*             checks validity of payload ptype                 */
 /****************************************************************/
 
-int nsys_netin_payvalid(struct nsys_payinfo marray[], int mlen, int fd) 
+int nsys_netin_payvalid(struct nsys_payinfo marray[], int mlen, 
+			int fd) 
 			 
 
 {
@@ -1225,7 +1292,7 @@ int nsys_netin_payvalid(struct nsys_payinfo marray[], int mlen, int fd)
 
   for (i = 0; i < mlen; i++)
     for (j = 0; j < NSYS_RTP_PAYSIZE; j++)
-      if (!strcmp(marray[i].name, nsys_payload_types[j].name))
+      if (!strcasecmp(marray[i].name, nsys_payload_types[j].name))
 	{
 	  if (nsys_msession || (marray[i].srate == -1) || 
 	      (marray[i].srate == nsys_payload_types[j].srate))
@@ -1271,7 +1338,7 @@ void nsys_netin_payset(struct nsys_source * sptr, struct nsys_payinfo marray[],
 
   for (i = 0; i < mlen; i++)
     for (j = 0; j < NSYS_RTP_PAYSIZE; j++)
-      if (!strcmp(marray[i].name, nsys_payload_types[j].name))
+      if (!strcasecmp(marray[i].name, nsys_payload_types[j].name))
 	{
 	  sptr->ptype = marray[i].ptype;
 	  sptr->pindex = nsys_payload_types[j].pindex;
