@@ -339,8 +339,8 @@ void ifoptconst(int passrate, tnode ** lptr)
     {
       currconstoptif++;
 
-      /* when we revisit, consider not sending passrate as */
-      /* the stateoptconst rate parameter for slower rates */
+      /* stateoptconst() rate parameters in the section below   */
+      /* should be semantically correct, but I may be mistaken. */
 
       /* pass for slower-rate i-rate statements */
 
@@ -351,7 +351,7 @@ void ifoptconst(int passrate, tnode ** lptr)
 	    {
 	      before = *bptr;
 	      if ((*bptr)->rate == IRATETYPE)
-		stateoptconst(passrate, bptr);
+		stateoptconst(IRATETYPE, bptr);
 	      if (before == (*bptr))
 		bptr = &((*bptr)->next);
 	    }
@@ -366,7 +366,7 @@ void ifoptconst(int passrate, tnode ** lptr)
 	    {
 	      before = *bptr;
 	      if ((*bptr)->rate == KRATETYPE)
-		stateoptconst(passrate, bptr);
+		stateoptconst(KRATETYPE, bptr);
 	      if (before == (*bptr))
 		bptr = &((*bptr)->next);
 	    }
@@ -457,10 +457,8 @@ void ifelseoptconst(int passrate, tnode ** lptr)
     {
       currconstoptif++;
 
-      /*************************************************************/
-      /* first the if block. when we revisit, consider not sending */
-      /* passrate as the stateoptconst parameter for slower rates  */
-      /*************************************************************/
+      /* stateoptconst() rate parameters in the section below   */
+      /* should be semantically correct, but I may be mistaken. */
 
       /* pass for slower-rate i-rate statements */
 
@@ -471,7 +469,7 @@ void ifelseoptconst(int passrate, tnode ** lptr)
 	    {
 	      before = *bptr;
 	      if ((*bptr)->rate == IRATETYPE)
-		stateoptconst(passrate, bptr);
+		stateoptconst(IRATETYPE, bptr);
 	      if (before == (*bptr))
 		bptr = &((*bptr)->next);
 	    }
@@ -486,7 +484,7 @@ void ifelseoptconst(int passrate, tnode ** lptr)
 	    {
 	      before = *bptr;
 	      if ((*bptr)->rate == KRATETYPE)
-		stateoptconst(passrate, bptr);
+		stateoptconst(KRATETYPE, bptr);
 	      if (before == (*bptr))
 		bptr = &((*bptr)->next);
 	    }
@@ -517,7 +515,7 @@ void ifelseoptconst(int passrate, tnode ** lptr)
 	    {
 	      before = *bptr;
 	      if ((*bptr)->rate == IRATETYPE)
-		stateoptconst(passrate, bptr);
+		stateoptconst(IRATETYPE, bptr);
 	      if (before == (*bptr))
 		bptr = &((*bptr)->next);
 	    }
@@ -532,7 +530,7 @@ void ifelseoptconst(int passrate, tnode ** lptr)
 	    {
 	      before = *bptr;
 	      if ((*bptr)->rate == KRATETYPE)
-		stateoptconst(passrate, bptr);
+		stateoptconst(KRATETYPE, bptr);
 	      if (before == (*bptr))
 		bptr = &((*bptr)->next);
 	    }
@@ -1510,8 +1508,8 @@ void userdefcollapse(int passrate, tnode * t_opcode, int allconst)
       before = before->next;
     }
 
-  /* optimize line-by-line, rate by rate. when we revisit, */
-  /* examine the use of passrate as stateoptconst rate     */
+  /* stateoptconst() rate parameters in the section below   */
+  /* should be semantically correct, but I may be mistaken. */
 
   for (ratetype = IRATETYPE; ratetype <= passrate; ratetype++)
     {
@@ -1524,7 +1522,7 @@ void userdefcollapse(int passrate, tnode * t_opcode, int allconst)
 	  currconstoptif = 0;
 	  currconstoptwhile = 0;
 	  if ((*tptr)->rate == ratetype)
-	    stateoptconst(passrate, tptr);
+	    stateoptconst(ratetype, tptr);
       
 	  if (before == (*tptr))
 	    tptr = &((*tptr)->next);
