@@ -707,6 +707,9 @@ void globalrefer(sigsym * iptr)
   tnode * optr = iptr->defnode->optr;
   int skip;
 
+  /* later consider adding currinstrument check, and removing */
+  /* currinstrument check below, for faster table accesses.   */
+
   skip = !strcmp(iptr->val,"startup");
   if (skip && reachableinstrexstart(iptr))
     {
@@ -732,7 +735,8 @@ void globalrefer(sigsym * iptr)
 		  gptr->tref->assigntot++;
 		  if (sptr->tref->assigntval)
 		    gptr->tref->assigntval++;
-		  if (currinstrument->vol > gptr->tref->finalinstr)
+		  if (currinstrument &&
+		      (currinstrument->vol > gptr->tref->finalinstr))
 		    gptr->tref->finalinstr = currinstrument->vol;
 		}
 	    }
