@@ -154,22 +154,30 @@ void make_instrpfields(tnode * t_identlist)
 {
   int symstate;
   int i = 0;
+  tnode * tptr = t_identlist;
 
-  while (t_identlist != NULL)
-    if (t_identlist->ttype == S_IDENT)
+  while (tptr != NULL)
+    if (tptr->ttype == S_IDENT)
       {
 	i++;
-	reservednames(t_identlist);
-	symstate = addsym(&locsymtable, t_identlist);
-	symcheck(symstate,t_identlist);
+	reservednames(tptr);
+	symstate = addsym(&locsymtable, tptr);
+	symcheck(symstate,tptr);
 	locsymtable->kind = K_PFIELD;
-	t_identlist->sptr = locsymtable;
-	t_identlist->width = locsymtable->width = 1;
-	t_identlist->rate = locsymtable->rate = IRATETYPE;
-	t_identlist = t_identlist->next;
+	tptr->sptr = locsymtable;
+	tptr->width = locsymtable->width = 1;
+	tptr->rate = locsymtable->rate = IRATETYPE;
+	tptr = tptr->next;
       }
     else
-      t_identlist = t_identlist->next;
+      tptr = tptr->next;
+
+  if (i > 255)
+    {
+      printf("Error: Instr or template has more than 255 parameters.\n\n");
+      showerrorplace(t_identlist->linenum, t_identlist->filename);
+    }
+
   if (i > numpfields)
     numpfields = i;
 
@@ -178,6 +186,7 @@ void make_instrpfields(tnode * t_identlist)
   
 }
 
+
 /***********************************************************************/
 /* make_miditag:    part of instr definition                           */
 /***********************************************************************/
@@ -289,6 +298,33 @@ tnode * make_opcodedecl(tnode * t_optype, tnode * t_ident, tnode * t_lp,
   retptr->sptr = reversetable(locsymtable);
   locsymtable = NULL;
 
+  if (t_optype->ttype != S_AOPCODE)
+    {
+      tptr = locopcodecalls;
+      while (tptr != NULL)
+	{
+	  if (coreopcodespecial(tptr))
+	    {
+	      printf("Error: Specialop %s may not appear in ", tptr->val);
+
+	      switch (t_optype->ttype) { 
+	      case S_IOPCODE:
+		printf("an iopcode definition.\n");
+		break;
+	      case S_KOPCODE:
+		printf("a kopcode definition.\n");
+		break;
+	      case S_OPCODE:
+		printf("a polymorphic opcode definition.\n");
+		break;
+	      }
+	      showerrorplace(tptr->optr->down->linenum, 
+			     tptr->optr->down->filename);
+	    }
+	  tptr = tptr->next;
+	}
+    }
+
   retptr->optr = locopcodecalls;
   locopcodecalls = NULL;
 
@@ -530,8 +566,9 @@ tnode * make_templatedecl(tnode * t_instrs,  tnode * t_miditags,
 	      while (mptr != NULL)
 		{
 		  if (mptr->ttype == S_EXPR)
-		    {
+		    {		      
 		      co_constcollapse(mptr);
+
 		      if (((mptr->down->ttype != S_INTGR) && (mptr->down->ttype != S_NUMBER))
 			  || (mptr->down->next != NULL))
 			{
@@ -850,14 +887,14 @@ tnode * make_routedef(tnode * t_route, tnode * t_lp, tnode * t_ident,
 
 tnode * make_senddef(tnode * t_send, tnode * t_lp, tnode * t_ident,
 		     tnode * t_sem1, tnode * t_exprlist, tnode * t_sem2,
-		     tnode * t_identlist, tnode * t_rp, tnode * t_sem3)
+		     tnode * t_namelist, tnode * t_rp, tnode * t_sem3)
 
 
 {
-  tnode * retptr;
-  tnode * ptr;
+  tnode * retptr, * ptr;
+  char * name;
 
-  if (t_identlist == NULL)
+  if (t_namelist == NULL)
     {      
       printf("Error: Send statement must reference a bus.\n\n");
       showerrorplace(t_send->linenum, t_send->filename);
@@ -867,35 +904,37 @@ tnode * make_senddef(tnode * t_send, tnode * t_lp, tnode * t_ident,
   t_sem1->next = make_tnode("<exprlist>",S_EXPRLIST);
   t_sem1->next->down = t_exprlist;
   t_sem1->next->next = t_sem2;
-  t_sem2->next = make_tnode("<identlist>",S_IDENTLIST);
-  t_sem2->next->down = t_identlist;
+  t_sem2->next = make_tnode("<namelist>",S_NAMELIST);
+  t_sem2->next->down = t_namelist;
   t_sem2->next->next = t_rp;
   t_rp->next = t_sem3;
 
   /* send statement defines buses - we log definitions below */
 
-  ptr = t_identlist;
+  ptr = t_namelist;
   while (ptr != NULL)
-    if (ptr->ttype == S_IDENT)
+    if (ptr->ttype == S_NAME)
       {
-	if ((addvsym(&busnametable,ptr->val,K_BUSNAME) != INSTALLED)
-	    &&(!strcmp(ptr->val,"output_bus")))
+	name = ptr->down->val;
+	if ((addvsym(&busnametable, name, K_BUSNAME) != INSTALLED)
+	    &&(!strcmp(name, "output_bus")))
 	  {
 	    printf("Error: Multiple definition of output_bus.\n\n");
-	    showerrorplace(ptr->linenum, ptr->filename);
+	    showerrorplace(ptr->down->linenum, ptr->down->filename);
 	  }
-	if (!strcmp(ptr->val,"output_bus"))
+	if (!strcmp(name,"output_bus"))
 	  {
 	    if (outputbusinstance != NULL)
 	      {
 		printf("Error: Output_bus sent to multiple instances.\n\n");
-		showerrorplace(ptr->linenum, ptr->filename);
+		showerrorplace(ptr->down->linenum, ptr->down->filename);
 	      }
 	    outputbusinstance = make_tnode(t_ident->val, S_INSTANCE);
 	    outputbusinstance->down = retptr;
 	  }
 	busnametable->defnode = retptr->down; /* send command creating bus */
-	ptr->sptr = getsym(&busnametable,ptr);
+	busnametable->width = 0;
+	ptr->sptr = getsym(&busnametable,ptr->down);
 	ptr = ptr->next;
       }
     else
