@@ -166,11 +166,7 @@ void globalparamcheck(void)
   if (outchannels <= 0)
     {
       if (outchannels == 0)
-	{
-	  printf("Warning: Ignoring global parameter outchannel 0 value, \n");
-	  printf("         setting value to 1 (note: technically not\n");
-	  printf("         compliant with ISO/IEC 14496-3 sec5).\n");
-	}
+	printf("Warning: Setting global parameter outchannel to 1.\n");
       outchannels = 1;
     }
 
@@ -246,18 +242,7 @@ void businitcheck(void)
 
 {  
   sigsym * sptr, * inbus;
-  tnode * tptr;
-
-  /**************************/
-  /* set buswidths to zero  */
-  /**************************/
-
-  sptr = busnametable;
-  while (sptr != NULL)
-    {
-      sptr->width = 0;
-      sptr = sptr->next;
-    }
+  tnode * tptr, * gptr;
 
   /********************************************************/
   /* add output bus to busnametable if not there already  */
@@ -312,6 +297,71 @@ void businitcheck(void)
       sptr->outputbus = 1;
       sptr = sptr->next;
     }
+
+  /***********************************/
+  /* record explicitly defined buses */
+  /***********************************/
+
+  gptr = groot;
+  while (gptr != NULL)
+    {
+      if (gptr->ttype == S_SENDDEF)
+	{
+	  tptr = gptr->down->next->next->next->next->next->next->down;
+
+	  while (tptr)
+	    {
+	      if ((tptr->ttype == S_NAME) && (tptr->down->next))
+		{
+		  switch (tptr->down->next->next->ttype) {
+		  case S_INTGR:
+		    if (tptr->sptr->width && 
+			(tptr->sptr->width != atoi(tptr->down->next->next->val)))
+		      {
+			printf("Error: Bus %s width declaration mismatch.\n",
+			       tptr->sptr->val);
+			showerrorplace(tptr->down->linenum, tptr->down->filename);
+		      }
+		    tptr->sptr->width = atoi(tptr->down->next->next->val);
+		    break;
+		  case S_INCHANNELS:
+		    if (!inchannels)
+		      {
+			printf("Error: Zero-width bus %s (via inchannels).\n",
+			       tptr->sptr->val);
+			showerrorplace(tptr->down->linenum, tptr->down->filename);
+		      }
+		    if (tptr->sptr->width && (tptr->sptr->width != inchannels))
+		      {
+			printf("Error: Bus %s width declaration mismatch.\n",
+			       tptr->sptr->val);
+			showerrorplace(tptr->down->linenum, tptr->down->filename);
+		      }
+		    tptr->sptr->width = inchannels;
+		    break;
+		  case S_OUTCHANNELS:
+		    if (!outchannels)
+		      {
+			printf("Error: Zero-width bus %s (via outchannels).\n",
+			       tptr->sptr->val);
+			showerrorplace(tptr->down->linenum, tptr->down->filename);
+		      }
+		    if (tptr->sptr->width && (tptr->sptr->width != outchannels))
+		      {
+			printf("Error: Bus %s width declaration mismatch.\n",
+			       tptr->sptr->val);
+			showerrorplace(tptr->down->linenum, tptr->down->filename);
+		      }
+		    tptr->sptr->width = outchannels;
+		    break;
+		  }
+		}
+	      tptr = tptr->next;
+	    }
+	}
+      gptr = gptr->next;
+    }
+
 }
 
 
@@ -335,7 +385,7 @@ void globalstatementcheck(void)
 
 {  
   int i;
-  tnode * tptr;
+  tnode * tptr, * iptr;
   tnode * gptr;
   sigsym * sptr;
 
@@ -354,7 +404,11 @@ void globalstatementcheck(void)
 		   gptr->down->next->next->val);
 	    showerrorplace(gptr->down->linenum,gptr->down->filename);
 	  }
+
+	i = 0;
+	iptr = NULL;
 	tptr = gptr->down->next->next->next->next->down; /* instr list */
+
 	while (tptr != NULL)
 	  {
 	    if (tptr->ttype == S_IDENT)
@@ -367,11 +421,30 @@ void globalstatementcheck(void)
 		    printf("Error: Undefined instrument in route statement.\n");
 		    showerrorplace(gptr->down->linenum,gptr->down->filename);
 		  }
+
+		/* state variables for instr width resolving */
+
+		if ((i++) == 0)
+		  iptr = tptr;
+		else
+		  if (iptr)
+		    iptr = (iptr->sptr == tptr->sptr) ? iptr : NULL;
+		
 		tptr->sptr->width = OUTCHANNELSWIDTH;
 		tptr->sptr->outputbus = 0;
 	      }
+
 	    tptr = tptr->next;
 	  }
+
+	/* if instr field suitable, and bus width known, resolve */
+
+	if (iptr && gptr->sptr->width && !(gptr->sptr->width % i))
+	  {
+	    iptr->sptr->width = gptr->sptr->width/i;
+	    iptr->sptr->outputbus = (gptr->sptr == outputbus);
+	  }
+
 	break;
       case S_SENDDEF:    
 
@@ -391,9 +464,6 @@ void globalstatementcheck(void)
 	      varupdate(tptr,&globalsymtable);
 	    tptr = tptr->next;
 	  }
-
-	/* identlist sptr's point to busnametable entries */
-
 	break;
       case S_SEQDEF:                 
 	tptr = gptr->down->next->next->down;
@@ -687,8 +757,10 @@ void instrorder(void)
   sigsym * outputbusinstr = NULL;
   tnode * sendptr;
   tnode * routeptr;
+  tnode * seqptr;
   tnode * blist;
   sigsym * lastinstr;
+  sigsym * iptr;
   int changed, passcount;
 
   /* first, take instrument being sent output_bus off list */
@@ -721,15 +793,15 @@ void instrorder(void)
 	  blist = sendptr->down->next->next->next->next->next->next->down;
 	  while (blist)
 	    {
-	      if ((blist->ttype==S_IDENT) &&          /* a bus that       */
-		  (strcmp(blist->val,"output_bus")))  /* isn't output_bus */
+	      if ((blist->ttype == S_NAME) &&               /* a bus that       */
+		  (strcmp(blist->down->val,"output_bus")))  /* isn't output_bus */
 		{         
 		  routeptr = groot;
 		  while (routeptr)
 		    {
 		      if ((routeptr->ttype == S_ROUTEDEF) &&
 			  (!strcmp(routeptr->down->next->next->val,
-				   blist->val)))
+				   blist->down->val)))
 			{
 			  /* found a route statement routing a bus sent by */
 			  /* the send statement */
@@ -745,7 +817,7 @@ void instrorder(void)
                                     sendptr->down->next->next->val),
 				    lastinstr);
 			}
-		      routeptr=routeptr->next;
+		      routeptr = routeptr->next;
 		    }
 		}
 	      blist = blist->next;
@@ -783,12 +855,12 @@ void instrorder(void)
     {
       changed = 0;
       passcount++;
-      sendptr = groot;
-      while (sendptr != NULL) 
+      seqptr = groot;
+      while (seqptr != NULL) 
 	{
-	  if (sendptr->ttype == S_SEQDEF)
+	  if (seqptr->ttype == S_SEQDEF)
 	    {
-	      blist = sendptr->down->next->next->down;
+	      blist = seqptr->down->next->next->down;
 	      while (blist->next != NULL)
 		{
 		  changed |= movebefore(&instrnametable,
@@ -797,22 +869,61 @@ void instrorder(void)
 		  blist = blist->next->next;
 		}
 	    }
-	  sendptr= sendptr->next;
+	  seqptr= seqptr->next;
 	}
     }
   if (changed)
     internalerror("postparse.c","end of instrorder()");
 
+  /* move startup back to start of list */
+
+  if (startupinstr && (startupinstr != instrnametable))
+    {
+      iptr = instrnametable;
+      while (iptr && iptr->next)
+	{
+	  if (iptr->next == startupinstr)
+	    {
+	      iptr->next = iptr->next->next;
+	      break;
+	    }
+	  iptr = iptr->next;
+	}
+      startupinstr->next = instrnametable;
+      instrnametable = startupinstr;
+    }
+
   /* set vol field to ordinal count in instrnametable */
 
-  lastinstr = instrnametable;
-  passcount = 0;
-  while (lastinstr)
+  iptr = instrnametable;
+  lastinstr = NULL;
+
+  while (iptr)
     {
-      lastinstr->vol = passcount++;
-      lastinstr = lastinstr->next;
+      if (reachableinstr(iptr))
+	{
+	  iptr->vol = numinstrnames++;
+	  lastinstr = iptr;
+	  iptr = iptr->next;
+	}
+      else
+	{
+	  if (lastinstr)
+	    {
+	      lastinstr->next = iptr->next;
+	      iptr->next = unusedinstrtable;
+	      unusedinstrtable = iptr;
+	      iptr = lastinstr->next;
+	    }
+	  else
+	    {
+	      iptr = instrnametable->next;
+	      instrnametable->next = unusedinstrtable;
+	      unusedinstrtable = instrnametable;
+	      instrnametable = iptr;
+	    }
+	}
     }
-  numinstrnames = passcount;
 
 }
 
@@ -893,7 +1004,7 @@ void instanceorder (void)
 
 
 extern void checkopcodeargswidth(tnode *);
-extern void buswidthcheck(sigsym *);
+extern void resolveinstrument(sigsym *);
 extern void instrbuswrite(sigsym *);
 extern int  instrinputwidth(tnode *);
 
@@ -916,10 +1027,10 @@ void buswidth(void)
       if (sptr->effects == 0)
 	{
 	  currinputwidth = 1;
-	  currinstrwidth = sptr->width;
+	  currinstrwidth = OUTCHANNELSWIDTH;
 	  widthupdate(sptr->defnode->down->
 		      next->next->next->next->next->next->next->next->down);
-	  buswidthcheck(sptr);
+	  resolveinstrument(sptr);
 	  checkopcodeargswidth(sptr->defnode->optr);
 	  sptr->width = currinstrwidth;
 	  instrbuswrite(sptr);
@@ -930,10 +1041,10 @@ void buswidth(void)
 	  while ((iptr != NULL) && (iptr->sptr == sptr))
 	    {
 	      currinputwidth = iptr->inwidth = instrinputwidth(iptr);
-	      currinstrwidth = sptr->width;
+	      currinstrwidth = OUTCHANNELSWIDTH;
 	      widthupdate(sptr->defnode->down->
 			  next->next->next->next->next->next->next->next->down);
-	      buswidthcheck(sptr);
+	      resolveinstrument(sptr);
 	      checkopcodeargswidth(sptr->defnode->optr);
 	      if (first)
 		{
@@ -945,10 +1056,11 @@ void buswidth(void)
 		{
 		  old = sptr->width;
 		  sptr->width = currinstrwidth;
-		  instrbuswrite(sptr);    /* better error message */
+		  instrbuswrite(sptr);   /* do first. to catch the common errors */
 		  if (old != sptr->width)
 		    {
-		      printf("Error: Instances of %s have different widths (%i and %i).\n",
+		      printf("Error: Instances of %s have different widths"
+			     " (%i and %i).\n",
 			   sptr->val, old, sptr->width);
 		      showerrorplace(iptr->down->down->linenum,
 				     iptr->down->down->filename);
@@ -966,11 +1078,16 @@ void buswidth(void)
     {
       if (sptr->width == 0)
 	{
+	  printf("Warning: Bus %s has undefined output width.\n"
+		 "         Sfront uses width of 1 (implementation-dependent).\n\n",
+		 sptr->val);
+
 	  sptr->width = 1;
 	}
       sptr->vol = i++;
       sptr = sptr->next;
     }
+
 }
 
 
@@ -1003,7 +1120,7 @@ void inbusinstall(void)
       tptr->width = 0;
       while (bptr != NULL)
 	{
-	  if (bptr->ttype == S_IDENT)
+	  if (bptr->ttype == S_NAME)
 	    {
 	      ibusptr->val = dupval(bptr->sptr->val);
 	      ibusptr->sptr = bptr->sptr;
@@ -1615,23 +1732,25 @@ void checkopcodeargswidth(tnode * tcall)
 }
 
 /****************************************************************/
-/*   check for errors in instrument bus width: in buswidth()    */
+/*        resolves instruments with undetermined widths         */
 /****************************************************************/
 
-void buswidthcheck(sigsym * sptr)
+void resolveinstrument(sigsym * sptr)
 
 {
+  /* use any fixed bus is routes from ... */
 
-  if ((sptr->outputbus)    &&        /* writes to output_bus */
-      (currinstrwidth>0)   &&        /* tree has a guess (even 1) */
-      (currinstrwidth!=sptr->width)) /* and its wrong */
-    {
-      printf("Error: Output width mismatch (outchannels).\n");
-      showerrorplace(sptr->defnode->down->linenum,
-		     sptr->defnode->down->filename);
-    }
-  if (currinstrwidth < 1)
-    {
+  if ((currinstrwidth <= UNKNOWN) && (sptr->width > UNKNOWN))
+    currinstrwidth = sptr->width;  
+
+  /* ... or else use default, and issue warning */
+
+  if (currinstrwidth <= UNKNOWN)
+    {	  
+      printf("Warning: Instrument %s has undefined output width.\n"
+	     "         Sfront uses width of 1 (implementation-dependent).\n\n",
+	     sptr->val);
+      currinstrwidth = 1;
     }
 }
 
@@ -1673,10 +1792,10 @@ void instrbuswrite(sigsym * iptr)
 	    }
 	  if (ready && found)
 	    {
-	      if (gptr->sptr->width <= 1)
+	      if (gptr->sptr->width == UNKNOWN)
 		gptr->sptr->width = width;
 	      else
-		if ((width != gptr->sptr->width)&&(width > 1))
+		if ((width != gptr->sptr->width) && (width > 1))
 		  {
 		    printf("Error: width mismatch (%i and %i) for bus %s.\n",
 			   width, gptr->sptr->width,
@@ -1684,13 +1803,23 @@ void instrbuswrite(sigsym * iptr)
 		    showerrorplace(gptr->down->linenum,
 				   gptr->down->filename);
 		  }
-		      
 	    }
 	}
       gptr = gptr->next;
     }
+
+  /* check width match for direct routes to output bus */
+
+  if ((iptr->outputbus) && (iptr->width > 1) && (iptr->width != outchannels))
+    {
+      printf("Error: Width mismatch (%i vs %i) for instr %s on the output_bus.\n",
+	     iptr->width, outchannels, iptr->val);
+      showerrorplace(iptr->defnode->down->linenum, 
+		     iptr->defnode->down->filename);
+    }
 }
 
+
 /****************************************************************/
 /*  given sequence rules, computes input width: in buswidth()   */
 /****************************************************************/
@@ -1705,7 +1834,7 @@ int instrinputwidth(tnode  * iptr)
 
   while (bptr != NULL)
     {
-      if (bptr->ttype == S_IDENT)
+      if (bptr->ttype == S_NAME)
 	ret += bptr->sptr->width;
       bptr = bptr->next;
     }
