@@ -1044,7 +1044,7 @@ tnode * scorelex (int scotype)
 
 {
   FILE * sfile = NULL;
-  int c,i,foundit;
+  int c,i,foundit, hashmark;
   char buf[LEXBUFSIZE] = {'\0'};
   tnode * scorelval;
 
@@ -1067,18 +1067,19 @@ tnode * scorelex (int scotype)
 
   scorelval = make_tnode("", S_BADCHAR);
 
-  /* delete whitespace and comments */
+  /* delete whitespace, hash-defines, and comments */
  
   foundit = 0;
   while (!foundit)
     {
       while ((isspace(c = getc(sfile)))&&(c != '\n'));
-      if (c != '/')
+      if ((c != '#') && (c != '/')) 
 	foundit = 1;
       else
 	{
+	  hashmark = (c == '#');
 	  c = getc(sfile);
-	  if (c != '/')
+	  if ((hashmark == 0) && (c != '/'))
 	    {
 	      ungetc(c,sfile);
 	      buf[0]='/'; buf[1]='\0';
