@@ -529,13 +529,23 @@ int standardcollapse(tnode * ident)
       ident->val = dupval(name);
       return CONSTANT;
     }
+  if (!(strcmp(ident->val,"inchan")))
+    {
+      ident->vol   = CONSTANT;
+      ident->ttype = S_INTGR;
+      ident->res = ASINT;
+      ident->rate = IRATETYPE;
+      sprintf(name,"%i", currinputwidth);
+      ident->val = dupval(name);
+      return CONSTANT;
+    }
   if (!(strcmp(ident->val,"outchan")))
     {
       ident->vol   = CONSTANT;
       ident->ttype = S_INTGR;
       ident->res = ASINT;
       ident->rate = IRATETYPE;
-      sprintf(name,"%i",outchannels);
+      sprintf(name,"%i", currinstrwidth);
       ident->val = dupval(name);
       return CONSTANT;
     }
@@ -658,6 +668,7 @@ void constbus(tnode ** ident, int group)
       bptr = bptr->next;
       gcount++;
     }
+
   fprintf(outfile," 0 ");      /* out of range */
 
   *ident = tptr->next->next->next;
@@ -832,9 +843,9 @@ void printstandardname(tnode ** ident)
   if (!(strcmp(tptr->val,"outchan")))
     {
       if (currintprint == ASFLOAT)
-	fprintf(outfile," %i.0F ", outchannels);
+	fprintf(outfile," %i.0F ", currinstrwidth);
       else
-	fprintf(outfile," %i ", outchannels);
+	fprintf(outfile," %i ", currinstrwidth);
       return;
     }
   if (!(strcmp(tptr->val,"time")))
