@@ -55,7 +55,7 @@ char * ratetostr(int rate)
 int truewidth(int width)
 
 {
-  if (width == INCHANNELSWIDTH)
+  if ((width == INCHANNELSWIDTH) || (width == CHANNELSWIDTH))
     return currinputwidth;
   if (width == OUTCHANNELSWIDTH)
     return currinstrwidth;
@@ -898,6 +898,33 @@ void opraterecurse(tnode * tptr)
     }
 }
 
+/************************************************************/
+/*       update wavetable rates for symrateupdate use       */
+/*    error checks occur in waveinitcheck() [wtparse.c]     */
+/************************************************************/
+
+void tablerateupdate(tnode * tptr)
+
+
+{
+  tnode * pvalptr;
+
+  while (tptr)
+    {
+      if (tptr->ttype == S_TABLE)
+	{
+	  pvalptr = tptr->down->next->next->next->next->next->down;
+	  if (pvalptr)
+	    {
+	      curropcoderate = ARATETYPE;
+	      currtreerate = UNKNOWN;
+	      rateupdate(pvalptr);
+	    }
+	}
+      tptr = tptr->next;
+    }
+}
+
 /************************************************************/
 /*           update rates of an instrument                  */
 /************************************************************/
@@ -905,11 +932,15 @@ void opraterecurse(tnode * tptr)
 void inrateupdate(sigsym * sptr)
 
 {
+  tablerateupdate(sptr->defnode->down->
+		  next->next->next->next->next->next->next->down);
+
   curropcoderate = ARATETYPE;
   currtreerate = UNKNOWN;
 
   rateupdate(sptr->defnode->down->
 	     next->next->next->next->next->next->next->next->down);
+
   opraterecurse(sptr->defnode->optr);
 }
 
@@ -1131,15 +1162,21 @@ void rateupdate(tnode * tbranch)
 		  cptr->next = NULL;
 		  rateupdate(cptr);
 		  cptr->next = bptr;
+
+		  /*
 		  if (cptr->rate == ARATETYPE)
 		    {
 		      printf("Error: Extend <expr> may not be arate.\n");
 		      showerrorplace(tptr->down->linenum, 
 				     tptr->down->filename);
 		    }
+		  */
+
 		  tptr->rate = cptr->rate;
 		  if (currtreerate > tptr->rate)  /* guard, outer opcodes, */
 		    tptr->rate = currtreerate;
+		  if (tptr->rate == ARATETYPE)
+		    tptr->rate = KRATETYPE;
 		}
 	      if ( (tptr->ttype == S_EXPR) && (tptr->optr != NULL) )
 		{
@@ -1209,15 +1246,6 @@ void rateupdate(tnode * tbranch)
 			  cptr = cptr->next;
 			}
 		      tptr->rate = tptr->optr->rate = tptr->sptr->rate;
-		      if (tptr->optr->special && (curropcoderate != ARATETYPE))
-			{
-			  printf("Error: Specialop %s used in a %s opcode.\n",
-				 tptr->down->val, ratetostr(curropcoderate));
-			  printf("       Use specialops only in aopcodes "
-				 "and instrs.\n"); 
-			  showerrorplace(tptr->down->linenum, 
-					 tptr->down->filename);
-			}
 		    }
 		  checkopcodeargsrate(tptr);
 		  if (tptr->rate > currtreerate)
