@@ -487,7 +487,7 @@ void ipasssasl(sigsym * sptr, char c, char * prefix)
   fprintf(outfile,"      sysidx->kbirth = kcycleidx;\n");
   fprintf(outfile,"      ninstr[nextstate].iline = sysidx;\n");
   fprintf(outfile,"      sysidx->time = (kcycleidx-1)*KTIME;\n");
-  nextstateupdate();
+  nextstateupdate(NULL);
   fprintf(outfile,"      %s_ipass(sysidx->nstate);\n",sptr->val);
   fprintf(outfile,"    }\n");
   fprintf(outfile,"   break;\n");
@@ -735,7 +735,7 @@ void ipasscore(sigsym * sptr, char c, char * s)
   fprintf(outfile,"      sysidx->kbirth = kcycleidx;\n");
   fprintf(outfile,"      ninstr[nextstate].iline = sysidx;\n");
   fprintf(outfile,"      sysidx->time = (kcycleidx-1)*KTIME;\n");
-  nextstateupdate();
+  nextstateupdate(NULL);
   fprintf(outfile,"      %s_ipass(sysidx->nstate);\n",sptr->val);
   fprintf(outfile,"    }\n");
   fprintf(outfile,"   break;\n");
@@ -743,6 +743,7 @@ void ipasscore(sigsym * sptr, char c, char * s)
     {
       fprintf(outfile,"   case PAUSED:\n");
       fprintf(outfile,"      sysidx->noteon = PLAYING;\n");
+      fprintf(outfile,"      sysidx->kbirth = kcycleidx;\n");
       fprintf(outfile,"   break;\n");
     }
   fprintf(outfile,"   default:\n");
@@ -920,7 +921,7 @@ void ipassinitpass(sigsym * sptr)
 		  sptr->val,i);
 	  fprintf(outfile,"  ninstr[nextstate].iline = &e_%s[%i];\n",
 		  sptr->val,i);
-	  nextstateupdate();
+	  nextstateupdate(NULL);
 	  fprintf(outfile,"   %s_ipass();\n", tptr->val);
 	  fprintf(outfile,"\n");
 	  i++;
