@@ -55,8 +55,8 @@ void opcodefunctions(void)
 
   if (globalopcodecalls != NULL)
     {
-      currinputwidth = 0;
-      currinstrwidth = 1;
+      currinputwidth = inchannels;
+      currinstrwidth = outchannels;
       currinstancename = curropcodeprefix = "GBL";
       currinstrument = NULL;  
       currinstance = NULL;
@@ -70,9 +70,7 @@ void opcodefunctions(void)
   sptr = instrnametable;
   while (sptr != NULL)
     {
-      if ((sptr->score)||(sptr->ascore)||(sptr->dyn)||(sptr->startup)||
-	  (sptr->midi)||(sptr->amidi)
-	  ||((cmidi || session) && sptr->miditag)||csasl)
+      if (reachableinstrexeff(sptr))
 	{
 	  currinputwidth = 1;              /* was zero */
 	  currinstrwidth = sptr->width;
@@ -281,16 +279,21 @@ void printopcall(tnode * tptr)
   char newpfix[STRSIZE];
   char newname[STRSIZE];
   char * istack, * pstack;
-  char * specfix;
+  char * specfix, * retvartype;
   tnode * fptr;  /* formal parameters */
   tnode * aptr;  /* actual parameters */
 
   specfix = "";
+  retvartype = "float";
+
   if (tptr->special)
     {
       currblockrate = ARATETYPE;
       if (currspecialrate == ARATETYPE)
-	specfix = "_spec";
+	{
+	  specfix = "_spec";
+	  retvartype = "void";
+	}
     }
   else
     currblockrate = tptr->rate;
@@ -306,10 +309,10 @@ void printopcall(tnode * tptr)
   /*  function header */
 
   if (currinstance == NULL)
-    fprintf(outfile,"\n\nfloat %s%s(struct ninstr_types * nstate)\n{\n",
-	    newname, specfix);
+    fprintf(outfile,"\n\n%s %s%s(struct ninstr_types * nstate)\n{\n",
+	    retvartype, newname, specfix);
   else
-    fprintf(outfile,"\n\nfloat %s%s(void)\n{\n",newname,specfix);
+    fprintf(outfile,"\n\n%s %s%s(void)\n{\n", retvartype,newname,specfix);
 
   /* local variables for core opcodes and tables */
 
@@ -359,19 +362,23 @@ extern void printoparraychildren(tnode *, int);
 void printoparraycall(tnode * tptr)
 
 {
-  char * specfix;
+  char * specfix, * retvartype;
   char * istack, * pstack;
   int optype;
 
   istack = currinstancename;
   pstack = curropcodeprefix;
 
+  retvartype = "float";
   specfix = "";
   if (tptr->special)
     {
       currblockrate = ARATETYPE;
       if (currspecialrate == ARATETYPE)
-	specfix = "_spec";
+	{
+	  retvartype = "void";
+	  specfix = "_spec";
+	}
     }
   else
     currblockrate = tptr->rate;
@@ -382,11 +389,11 @@ void printoparraycall(tnode * tptr)
 
   if (currinstance == NULL)
     fprintf(outfile,
-	    "\n\nfloat %s__sym_%s%i%s(struct ninstr_types * nstate)\n{\n",
-	    curropcodeprefix, tptr->val, tptr->arrayidx,specfix);
+	    "\n\n%s %s__sym_%s%i%s(struct ninstr_types * nstate)\n{\n",
+	    retvartype, curropcodeprefix, tptr->val, tptr->arrayidx,specfix);
   else
-    fprintf(outfile, "\n\nfloat %s__sym_%s%i%s(void)\n{\n", 
-	    curropcodeprefix, tptr->val, tptr->arrayidx, specfix);
+    fprintf(outfile, "\n\n%s %s__sym_%s%i%s(void)\n{\n", 
+	    retvartype, curropcodeprefix, tptr->val, tptr->arrayidx, specfix);
   printlocals(tptr);
 
   /*****************************************/
@@ -599,13 +606,13 @@ void printopargs(tnode * tcall, tnode * aptr, tnode * fptr,
 		  /* signal variables */
 
 		  currarrayindex = 0;
-		  if (fptr->width > 1)
+		  if (truewidth(fptr->width) > 1)
 		    currscalarflag = 0;
 		  else
 		    currscalarflag = 1;
-		  while (currarrayindex < fptr->width)
+		  while (currarrayindex < truewidth(fptr->width))
 		    {
-		      if (fptr->width == 1)
+		      if (truewidth(fptr->width) == 1)
 			{
 			  if (core)
 			    {
