@@ -56,11 +56,9 @@ void toptree(int mode)
   sptr = instrnametable;
   while (sptr != NULL)
     {
-      if ((sptr->score)||(sptr->ascore)||(sptr->dyn)||(sptr->startup)
-	  ||(sptr->midi)||(sptr->amidi)
-	  ||((cmidi || session) && sptr->miditag)||csasl)
+      if (reachableinstrexeff(sptr))
 	{
-	  currinputwidth = 1;   /* was zero */
+	  currinputwidth = 1;
 	  currinstrwidth = sptr->width;
 	  curropcodeprefix = currinstancename = sptr->val;
 	  currinstrument = sptr;
@@ -650,8 +648,8 @@ void printsystemsinit(void)
       fprintf(outfile,"   startup_ipass(u_startup[0].nstate);\n");
     }
   iptr = globalsymtable;
-  currinputwidth = 0;
-  currinstrwidth = 1;
+  currinputwidth = inchannels;
+  currinstrwidth = outchannels;
   curropcodeprefix = currinstancename = "GBL";
   currinstrument = NULL;  
   currinstance = NULL;
@@ -723,8 +721,8 @@ void printeffectsinit(void)
 			}
 		      fprintf(outfile,"e_%s[%i].p[%s_%s] = ",sptr->val,i,
 			      instptr->val, pptr->val);
-		      currinputwidth = 0;
-		      currinstrwidth = 1;
+		      currinputwidth = inchannels;
+		      currinstrwidth = outchannels;
 		      widthupdate(pvalptr);
 		      curropcoderate = ARATETYPE;
 		      currtreerate = UNKNOWN;
