@@ -1198,101 +1198,113 @@ void controllibrary(void)
     {
       fprintf(outfile,"#define CSYS_MIDI_NUMCHAN  16\n\n");
 
-      fprintf(outfile,"#define CSYS_MIDI_SPECIAL  0x70\n");
-      fprintf(outfile,"#define CSYS_MIDI_NOOP     0x70\n");
-      fprintf(outfile,"#define CSYS_MIDI_NEWTEMPO 0x71\n");
-      fprintf(outfile,"#define CSYS_MIDI_ENDTIME  0x72\n");
-      fprintf(outfile,"#define CSYS_MIDI_POWERUP  0x73\n");
-      fprintf(outfile,"#define CSYS_MIDI_NOTEOFF  0x80\n");
-      fprintf(outfile,"#define CSYS_MIDI_NOTEON   0x90\n");
-      fprintf(outfile,"#define CSYS_MIDI_PTOUCH   0xA0\n");
-      fprintf(outfile,"#define CSYS_MIDI_CC       0xB0\n");
-      fprintf(outfile,"#define CSYS_MIDI_PROGRAM  0xC0\n");
-      fprintf(outfile,"#define CSYS_MIDI_CTOUCH   0xD0\n");
-      fprintf(outfile,"#define CSYS_MIDI_WHEEL    0xE0\n");
-      fprintf(outfile,"#define CSYS_MIDI_SYSTEM   0xF0\n\n");
+      fprintf(outfile,"#define CSYS_MIDI_SPECIAL  0x70u\n");
+      fprintf(outfile,"#define CSYS_MIDI_NOOP     0x70u\n");
+      fprintf(outfile,"#define CSYS_MIDI_NEWTEMPO 0x71u\n");
+      fprintf(outfile,"#define CSYS_MIDI_ENDTIME  0x72u\n");
+      fprintf(outfile,"#define CSYS_MIDI_POWERUP  0x73u\n");
+      fprintf(outfile,"#define CSYS_MIDI_TSTART   0x7Du\n");
+      fprintf(outfile,"#define CSYS_MIDI_MANUEX   0x7Du\n");
+      fprintf(outfile,"#define CSYS_MIDI_MVOLUME  0x7Eu\n");
+      fprintf(outfile,"#define CSYS_MIDI_GMRESET  0x7Fu\n");
+      fprintf(outfile,"#define CSYS_MIDI_NOTEOFF  0x80u\n");
+      fprintf(outfile,"#define CSYS_MIDI_NOTEON   0x90u\n");
+      fprintf(outfile,"#define CSYS_MIDI_PTOUCH   0xA0u\n");
+      fprintf(outfile,"#define CSYS_MIDI_CC       0xB0u\n");
+      fprintf(outfile,"#define CSYS_MIDI_PROGRAM  0xC0u\n");
+      fprintf(outfile,"#define CSYS_MIDI_CTOUCH   0xD0u\n");
+      fprintf(outfile,"#define CSYS_MIDI_WHEEL    0xE0u\n");
+      fprintf(outfile,"#define CSYS_MIDI_SYSTEM   0xF0u\n\n");
 
-      fprintf(outfile,"#define CSYS_MIDI_SYSTEM_SYSEX_START  0xF0\n");
-      fprintf(outfile,"#define CSYS_MIDI_SYSTEM_QFRAME       0xF1\n");
-      fprintf(outfile,"#define CSYS_MIDI_SYSTEM_SONG_PP      0xF2\n"); 
-      fprintf(outfile,"#define CSYS_MIDI_SYSTEM_SONG_SELECT  0xF3\n");
-      fprintf(outfile,"#define CSYS_MIDI_SYSTEM_UNUSED1      0xF4\n");
-      fprintf(outfile,"#define CSYS_MIDI_SYSTEM_UNUSED2      0xF5\n");
-      fprintf(outfile,"#define CSYS_MIDI_SYSTEM_TUNE_REQUEST 0xF6\n");
-      fprintf(outfile,"#define CSYS_MIDI_SYSTEM_SYSEX_END    0xF7\n");
-      fprintf(outfile,"#define CSYS_MIDI_SYSTEM_CLOCK        0xF8\n");
-      fprintf(outfile,"#define CSYS_MIDI_SYSTEM_TICK         0xF9\n");
-      fprintf(outfile,"#define CSYS_MIDI_SYSTEM_START        0xFA\n");
-      fprintf(outfile,"#define CSYS_MIDI_SYSTEM_CONTINUE     0xFB\n");
-      fprintf(outfile,"#define CSYS_MIDI_SYSTEM_STOP         0xFC\n");
-      fprintf(outfile,"#define CSYS_MIDI_SYSTEM_UNUSED3      0xFD\n");
-      fprintf(outfile,"#define CSYS_MIDI_SYSTEM_SENSE        0xFE\n");
-      fprintf(outfile,"#define CSYS_MIDI_SYSTEM_RESET        0xFF\n\n");
+      fprintf(outfile,"#define CSYS_MIDI_SYSTEM_SYSEX_START  0xF0u\n");
+      fprintf(outfile,"#define CSYS_MIDI_SYSTEM_QFRAME       0xF1u\n");
+      fprintf(outfile,"#define CSYS_MIDI_SYSTEM_SONG_PP      0xF2u\n"); 
+      fprintf(outfile,"#define CSYS_MIDI_SYSTEM_SONG_SELECT  0xF3u\n");
+      fprintf(outfile,"#define CSYS_MIDI_SYSTEM_UNUSED1      0xF4u\n");
+      fprintf(outfile,"#define CSYS_MIDI_SYSTEM_UNUSED2      0xF5u\n");
+      fprintf(outfile,"#define CSYS_MIDI_SYSTEM_TUNE_REQUEST 0xF6u\n");
+      fprintf(outfile,"#define CSYS_MIDI_SYSTEM_SYSEX_END    0xF7u\n");
+      fprintf(outfile,"#define CSYS_MIDI_SYSTEM_CLOCK        0xF8u\n");
+      fprintf(outfile,"#define CSYS_MIDI_SYSTEM_TICK         0xF9u\n");
+      fprintf(outfile,"#define CSYS_MIDI_SYSTEM_START        0xFAu\n");
+      fprintf(outfile,"#define CSYS_MIDI_SYSTEM_CONTINUE     0xFBu\n");
+      fprintf(outfile,"#define CSYS_MIDI_SYSTEM_STOP         0xFCu\n");
+      fprintf(outfile,"#define CSYS_MIDI_SYSTEM_UNUSED3      0xFDu\n");
+      fprintf(outfile,"#define CSYS_MIDI_SYSTEM_SENSE        0xFEu\n");
+      fprintf(outfile,"#define CSYS_MIDI_SYSTEM_RESET        0xFFu\n\n");
       
-      fprintf(outfile,"#define CSYS_MIDI_CC_BANKSELECT_MSB  0x00\n"); 
-      fprintf(outfile,"#define CSYS_MIDI_CC_MODWHEEL_MSB    0x01\n");    
-      fprintf(outfile,"#define CSYS_MIDI_CC_BREATHCNTRL_MSB 0x02\n");    
-      fprintf(outfile,"#define CSYS_MIDI_CC_FOOTCNTRL_MSB   0x04\n"); 
-      fprintf(outfile,"#define CSYS_MIDI_CC_PORTAMENTO_MSB  0x05\n");    
-      fprintf(outfile,"#define CSYS_MIDI_CC_DATAENTRY_MSB   0x06\n"); 
-      fprintf(outfile,"#define CSYS_MIDI_CC_CHANVOLUME_MSB  0x07\n");    
-      fprintf(outfile,"#define CSYS_MIDI_CC_BALANCE_MSB     0x08\n");    
-      fprintf(outfile,"#define CSYS_MIDI_CC_PAN_MSB         0x0A\n");    
-      fprintf(outfile,"#define CSYS_MIDI_CC_EXPRESSION_MSB  0x0B\n");    
-      fprintf(outfile,"#define CSYS_MIDI_CC_EFFECT1_MSB     0x0C\n");    
-      fprintf(outfile,"#define CSYS_MIDI_CC_EFFECT2_MSB     0x0D\n");    
-      fprintf(outfile,"#define CSYS_MIDI_CC_GEN1_MSB        0x10\n");    
-      fprintf(outfile,"#define CSYS_MIDI_CC_GEN2_MSB        0x11\n");    
-      fprintf(outfile,"#define CSYS_MIDI_CC_GEN3_MSB        0x12\n");   
-      fprintf(outfile,"#define CSYS_MIDI_CC_GEN4_MSB        0x13\n"); 
-      fprintf(outfile,"#define CSYS_MIDI_CC_BANKSELECT_LSB  0x20\n");    
-      fprintf(outfile,"#define CSYS_MIDI_CC_MODWHEEL_LSB    0x21\n");       
-      fprintf(outfile,"#define CSYS_MIDI_CC_BREATHCNTRL_LSB 0x22\n");       
-      fprintf(outfile,"#define CSYS_MIDI_CC_FOOTCNTRL_LSB   0x24\n");    
-      fprintf(outfile,"#define CSYS_MIDI_CC_PORTAMENTO_LSB  0x25\n");       
-      fprintf(outfile,"#define CSYS_MIDI_CC_DATAENTRY_LSB   0x26\n");    
-      fprintf(outfile,"#define CSYS_MIDI_CC_CHANVOLUME_LSB  0x27\n");       
-      fprintf(outfile,"#define CSYS_MIDI_CC_BALANCE_LSB     0x28\n");    
-      fprintf(outfile,"#define CSYS_MIDI_CC_PAN_LSB         0x2A\n");   
-      fprintf(outfile,"#define CSYS_MIDI_CC_EXPRESSION_LSB  0x2B\n");   
-      fprintf(outfile,"#define CSYS_MIDI_CC_EFFECT1_LSB     0x2C\n");   
-      fprintf(outfile,"#define CSYS_MIDI_CC_EFFECT2_LSB     0x2D\n");   
-      fprintf(outfile,"#define CSYS_MIDI_CC_GEN1_LSB        0x30\n");   
-      fprintf(outfile,"#define CSYS_MIDI_CC_GEN2_LSB        0x31\n");   
-      fprintf(outfile,"#define CSYS_MIDI_CC_GEN3_LSB        0x32\n");  
-      fprintf(outfile,"#define CSYS_MIDI_CC_GEN4_LSB        0x33\n");
-      fprintf(outfile,"#define CSYS_MIDI_CC_SUSTAIN         0x40\n");        
-      fprintf(outfile,"#define CSYS_MIDI_CC_PORTAMENTO      0x41\n");   
-      fprintf(outfile,"#define CSYS_MIDI_CC_SUSTENUTO       0x42\n"); 
-      fprintf(outfile,"#define CSYS_MIDI_CC_SOFTPEDAL       0x43\n"); 
-      fprintf(outfile,"#define CSYS_MIDI_CC_LEGATO          0x44\n");
-      fprintf(outfile,"#define CSYS_MIDI_CC_HOLD2           0x45\n");
-      fprintf(outfile,"#define CSYS_MIDI_CC_SOUNDCONTROL1   0x46\n");
-      fprintf(outfile,"#define CSYS_MIDI_CC_SOUNDCONTROL2   0x47\n");
-      fprintf(outfile,"#define CSYS_MIDI_CC_SOUNDCONTROL3   0x48\n");
-      fprintf(outfile,"#define CSYS_MIDI_CC_SOUNDCONTROL4   0x49\n");
-      fprintf(outfile,"#define CSYS_MIDI_CC_SOUNDCONTROL5   0x4A\n");
-      fprintf(outfile,"#define CSYS_MIDI_CC_SOUNDCONTROL6   0x4B\n");
-      fprintf(outfile,"#define CSYS_MIDI_CC_SOUNDCONTROL7   0x4C\n");
-      fprintf(outfile,"#define CSYS_MIDI_CC_SOUNDCONTROL8   0x4D\n");
-      fprintf(outfile,"#define CSYS_MIDI_CC_SOUNDCONTROL9   0x4E\n");
-      fprintf(outfile,"#define CSYS_MIDI_CC_SOUNDCONTROL10  0x4F\n");
-      fprintf(outfile,"#define CSYS_MIDI_CC_GEN5            0x50\n");   
-      fprintf(outfile,"#define CSYS_MIDI_CC_GEN6            0x51\n");   
-      fprintf(outfile,"#define CSYS_MIDI_CC_GEN7            0x52\n");  
-      fprintf(outfile,"#define CSYS_MIDI_CC_GEN8            0x53\n");
-      fprintf(outfile,"#define CSYS_MIDI_CC_PORTAMENTOSRC   0x54\n");
-      fprintf(outfile,"#define CSYS_MIDI_CC_EFFECT1DEPTH    0x5B\n");
-      fprintf(outfile,"#define CSYS_MIDI_CC_EFFECT2DEPTH    0x5C\n");
-      fprintf(outfile,"#define CSYS_MIDI_CC_EFFECT3DEPTH    0x5D\n");
-      fprintf(outfile,"#define CSYS_MIDI_CC_EFFECT4DEPTH    0x5E\n");
-      fprintf(outfile,"#define CSYS_MIDI_CC_EFFECT5DEPTH    0x5F\n");
-      fprintf(outfile,"#define CSYS_MIDI_CC_DATAENTRYPLUS   0x60\n");
-      fprintf(outfile,"#define CSYS_MIDI_CC_DATAENTRYMINUS  0x61\n");
-      fprintf(outfile,"#define CSYS_MIDI_CC_ALLSOUNDOFF     0x78\n");
-      fprintf(outfile,"#define CSYS_MIDI_CC_RESETALLCONTROL 0x79\n");
-      fprintf(outfile,"#define CSYS_MIDI_CC_LOCALCONTROL    0x7A\n");
-      fprintf(outfile,"#define CSYS_MIDI_CC_ALLNOTESOFF     0x7B\n");
+      fprintf(outfile,"#define CSYS_MIDI_CC_BANKSELECT_MSB  0x00u\n"); 
+      fprintf(outfile,"#define CSYS_MIDI_CC_MODWHEEL_MSB    0x01u\n");    
+      fprintf(outfile,"#define CSYS_MIDI_CC_BREATHCNTRL_MSB 0x02u\n");    
+      fprintf(outfile,"#define CSYS_MIDI_CC_FOOTCNTRL_MSB   0x04u\n"); 
+      fprintf(outfile,"#define CSYS_MIDI_CC_PORTAMENTO_MSB  0x05u\n");    
+      fprintf(outfile,"#define CSYS_MIDI_CC_DATAENTRY_MSB   0x06u\n"); 
+      fprintf(outfile,"#define CSYS_MIDI_CC_CHANVOLUME_MSB  0x07u\n");    
+      fprintf(outfile,"#define CSYS_MIDI_CC_BALANCE_MSB     0x08u\n");    
+      fprintf(outfile,"#define CSYS_MIDI_CC_PAN_MSB         0x0Au\n");    
+      fprintf(outfile,"#define CSYS_MIDI_CC_EXPRESSION_MSB  0x0Bu\n");    
+      fprintf(outfile,"#define CSYS_MIDI_CC_EFFECT1_MSB     0x0Cu\n");    
+      fprintf(outfile,"#define CSYS_MIDI_CC_EFFECT2_MSB     0x0Du\n");    
+      fprintf(outfile,"#define CSYS_MIDI_CC_GEN1_MSB        0x10u\n");    
+      fprintf(outfile,"#define CSYS_MIDI_CC_GEN2_MSB        0x11u\n");    
+      fprintf(outfile,"#define CSYS_MIDI_CC_GEN3_MSB        0x12u\n");   
+      fprintf(outfile,"#define CSYS_MIDI_CC_GEN4_MSB        0x13u\n"); 
+      fprintf(outfile,"#define CSYS_MIDI_CC_BANKSELECT_LSB  0x20u\n");    
+      fprintf(outfile,"#define CSYS_MIDI_CC_MODWHEEL_LSB    0x21u\n");       
+      fprintf(outfile,"#define CSYS_MIDI_CC_BREATHCNTRL_LSB 0x22u\n");       
+      fprintf(outfile,"#define CSYS_MIDI_CC_FOOTCNTRL_LSB   0x24u\n");    
+      fprintf(outfile,"#define CSYS_MIDI_CC_PORTAMENTO_LSB  0x25u\n");       
+      fprintf(outfile,"#define CSYS_MIDI_CC_DATAENTRY_LSB   0x26u\n");    
+      fprintf(outfile,"#define CSYS_MIDI_CC_CHANVOLUME_LSB  0x27u\n");       
+      fprintf(outfile,"#define CSYS_MIDI_CC_BALANCE_LSB     0x28u\n");    
+      fprintf(outfile,"#define CSYS_MIDI_CC_PAN_LSB         0x2Au\n");   
+      fprintf(outfile,"#define CSYS_MIDI_CC_EXPRESSION_LSB  0x2Bu\n");   
+      fprintf(outfile,"#define CSYS_MIDI_CC_EFFECT1_LSB     0x2Cu\n");   
+      fprintf(outfile,"#define CSYS_MIDI_CC_EFFECT2_LSB     0x2Du\n");   
+      fprintf(outfile,"#define CSYS_MIDI_CC_GEN1_LSB        0x30u\n");   
+      fprintf(outfile,"#define CSYS_MIDI_CC_GEN2_LSB        0x31u\n");   
+      fprintf(outfile,"#define CSYS_MIDI_CC_GEN3_LSB        0x32u\n");  
+      fprintf(outfile,"#define CSYS_MIDI_CC_GEN4_LSB        0x33u\n");
+      fprintf(outfile,"#define CSYS_MIDI_CC_SUSTAIN         0x40u\n");        
+      fprintf(outfile,"#define CSYS_MIDI_CC_PORTAMENTO      0x41u\n");   
+      fprintf(outfile,"#define CSYS_MIDI_CC_SUSTENUTO       0x42u\n"); 
+      fprintf(outfile,"#define CSYS_MIDI_CC_SOFTPEDAL       0x43u\n"); 
+      fprintf(outfile,"#define CSYS_MIDI_CC_LEGATO          0x44u\n");
+      fprintf(outfile,"#define CSYS_MIDI_CC_HOLD2           0x45u\n");
+      fprintf(outfile,"#define CSYS_MIDI_CC_SOUNDCONTROL1   0x46u\n");
+      fprintf(outfile,"#define CSYS_MIDI_CC_SOUNDCONTROL2   0x47u\n");
+      fprintf(outfile,"#define CSYS_MIDI_CC_SOUNDCONTROL3   0x48u\n");
+      fprintf(outfile,"#define CSYS_MIDI_CC_SOUNDCONTROL4   0x49u\n");
+      fprintf(outfile,"#define CSYS_MIDI_CC_SOUNDCONTROL5   0x4Au\n");
+      fprintf(outfile,"#define CSYS_MIDI_CC_SOUNDCONTROL6   0x4Bu\n");
+      fprintf(outfile,"#define CSYS_MIDI_CC_SOUNDCONTROL7   0x4Cu\n");
+      fprintf(outfile,"#define CSYS_MIDI_CC_SOUNDCONTROL8   0x4Du\n");
+      fprintf(outfile,"#define CSYS_MIDI_CC_SOUNDCONTROL9   0x4Eu\n");
+      fprintf(outfile,"#define CSYS_MIDI_CC_SOUNDCONTROL10  0x4Fu\n");
+      fprintf(outfile,"#define CSYS_MIDI_CC_GEN5            0x50u\n");   
+      fprintf(outfile,"#define CSYS_MIDI_CC_GEN6            0x51u\n");   
+      fprintf(outfile,"#define CSYS_MIDI_CC_GEN7            0x52u\n");  
+      fprintf(outfile,"#define CSYS_MIDI_CC_GEN8            0x53u\n");
+      fprintf(outfile,"#define CSYS_MIDI_CC_PORTAMENTOSRC   0x54u\n");
+      fprintf(outfile,"#define CSYS_MIDI_CC_EFFECT1DEPTH    0x5Bu\n");
+      fprintf(outfile,"#define CSYS_MIDI_CC_EFFECT2DEPTH    0x5Cu\n");
+      fprintf(outfile,"#define CSYS_MIDI_CC_EFFECT3DEPTH    0x5Du\n");
+      fprintf(outfile,"#define CSYS_MIDI_CC_EFFECT4DEPTH    0x5Eu\n");
+      fprintf(outfile,"#define CSYS_MIDI_CC_EFFECT5DEPTH    0x5Fu\n");
+      fprintf(outfile,"#define CSYS_MIDI_CC_DATAENTRYPLUS   0x60u\n");
+      fprintf(outfile,"#define CSYS_MIDI_CC_DATAENTRYMINUS  0x61u\n");
+      fprintf(outfile,"#define CSYS_MIDI_CC_NRPN_LSB        0x62u\n");
+      fprintf(outfile,"#define CSYS_MIDI_CC_NRPN_MSB        0x63u\n");
+      fprintf(outfile,"#define CSYS_MIDI_CC_RPN_LSB         0x64u\n");
+      fprintf(outfile,"#define CSYS_MIDI_CC_RPN_MSB         0x65u\n");
+      fprintf(outfile,"#define CSYS_MIDI_CC_ALLSOUNDOFF     0x78u\n");
+      fprintf(outfile,"#define CSYS_MIDI_CC_RESETALLCONTROL 0x79u\n");
+      fprintf(outfile,"#define CSYS_MIDI_CC_LOCALCONTROL    0x7Au\n");
+      fprintf(outfile,"#define CSYS_MIDI_CC_ALLNOTESOFF     0x7Bu\n");
+      fprintf(outfile,"#define CSYS_MIDI_CC_OMNI_OFF        0x7Cu\n");
+      fprintf(outfile,"#define CSYS_MIDI_CC_OMNI_ON         0x7Du\n");
+      fprintf(outfile,"#define CSYS_MIDI_CC_MONOMODE        0x7Eu\n");
+      fprintf(outfile,"#define CSYS_MIDI_CC_POLYMODE        0x7Fu\n");
       fprintf(outfile,"\n");
     }
 
@@ -3390,6 +3402,7 @@ void printcdriverlists(int * instrnum)
       printpresetinit("end");
       printpresetinit("next");
 
+      fprintf(outfile,"#define CSYS_MAXCINCHAN %i\n\n", cinmaxchan);
       fprintf(outfile,"#define CSYS_MAXEXTCHAN %i\n\n", cmaxchan);
       printextchaninit("first");
       printextchaninit("last");
