@@ -300,18 +300,19 @@ void createconstanttable(sigsym * ident, char * prefix, int mode)
     mz(lc); sprintf(z[lc++],
 		    "NT(%s_%s).t = (float *)(memcpy"
 		    "(malloc(%i), table_%s_%s, %i));",
-		    prefix, ident->val, sizeof(float)*(size+1),
-		    currinstancename, ident->val, sizeof(float)*(size+1));
+		    prefix, ident->val, (int)(sizeof(float)*(size+1)),
+		    currinstancename, ident->val, 
+		    (int)(sizeof(float)*(size+1)));
     mz(lc); sprintf(z[lc++], "NT(%s_%s).llmem = 1;", prefix, ident->val);
     break;
   case S_OPCODE:
     mz(lc); sprintf(z[lc++],
 		    "NT(%s_%s).t = (float *)(memcpy"
 		    "(malloc(%i), %s_%s%i_%s, %i));",
-		    prefix, ident->val, sizeof(float)*(size+1),
+		    prefix, ident->val, (int)(sizeof(float)*(size+1)),
 		    wavetablenameprefix(curropcodestack, "table_"),
 		    curropcodeinstance->val, curropcodeinstance->arrayidx,
-		    ident->val, sizeof(float)*(size+1));
+		    ident->val, (int)(sizeof(float)*(size+1)));
     mz(lc); sprintf(z[lc++], "NT(%s_%s).llmem = 1;", prefix, ident->val);
     break;
   default:
@@ -347,7 +348,7 @@ void printtablestring(sigsym * sptr, char * varname)
   if (hexstrings)
     {
       fprintf(outfile,"char %s[%i]",
-	      varname, sizeof(float)*(sptr->defnode->arrayidx + 1) + 1);
+	      varname, (int)(sizeof(float)*(sptr->defnode->arrayidx + 1) + 1));
 
       if (sptr->consval == NULL)
 	fprintf(outfile,";\n\n");
@@ -574,6 +575,7 @@ int cubicconstcheck(tnode * tptr, tnode * nsl, int size, float ** ret)
   char newval[STRSIZE];
   int i = 1;
   int x, oldx, infl, oldinfl;
+  float x_f, oldx_f, infl_f, oldinfl_f;
   float y, oldy, oldery;
   tnode * nptr = tptr;
   float * fptr;
@@ -582,13 +584,17 @@ int cubicconstcheck(tnode * tptr, tnode * nsl, int size, float ** ret)
   float Q, R, S, T, a, b, c, d;
 
   x = oldx = infl = oldinfl = -1;
+  x_f = oldx_f = infl_f = oldinfl_f = -1.0F;
 
   while (tptr != NULL)
     {
       switch (i & 3) {
       case 1:
 	oldinfl = infl;
+	oldinfl_f = (float) oldinfl;
 	infl = (int) make_long(tptr->down);
+	infl_f = (float) infl;
+
 	if (tptr->down->ttype == S_NUMBER)
 	  {
 	    tptr->res = tptr->down->res = ASINT;
@@ -604,28 +610,29 @@ int cubicconstcheck(tnode * tptr, tnode * nsl, int size, float ** ret)
 	  }	
 	if (i > 1)
 	  {
-	    Q = oldinfl*oldinfl*oldinfl - x*x*x - 3.0F*x*x*(oldinfl-x);
+	    Q = (oldinfl_f*oldinfl_f*oldinfl_f - x_f*x_f*x_f 
+		 - 3.0F*x_f*x_f*(oldinfl_f - x_f));
 	    if (Q == 0.0F)
 	      {     
 		printf("Error: No cubic solution for these parameters.\n");
 		printtableproblem(tptr, nsl);
 	      }
 
-	    R = oldinfl*oldinfl - x*x - 2.0F*x*(oldinfl-x);
+	    R = oldinfl_f*oldinfl_f - x_f*x_f - 2.0F*x_f*(oldinfl_f - x_f);
 	    if (R == 0.0F)
 	      {     
 		printf("Error: No cubic solution for these parameters.\n");
 		printtableproblem(tptr, nsl);
 	      }
       
-	    S = x*x*x - infl*infl*infl - 3.0F*x*x*(x-infl);
+	    S = x_f*x_f*x_f - infl_f*infl_f*infl_f - 3.0F*x_f*x_f*(x_f - infl_f);
 	    if (S == 0.0F)
 	      {     
 		printf("Error: No cubic solution for these parameters.\n");
 		printtableproblem(tptr, nsl);
 	      }
 	    
-	    T = x*x - infl*infl - 2.0F*x*(x-infl);
+	    T = x_f*x_f - infl_f*infl_f - 2.0F*x_f*(x_f - infl_f);
 	    if ((T == 0.0F) || ((Q/R) == (S/T)))
 	      {     
 		printf("Error: No cubic solution for these parameters.\n");
@@ -636,7 +643,9 @@ int cubicconstcheck(tnode * tptr, tnode * nsl, int size, float ** ret)
 	break;
       case 3:
 	oldx = x;
+	oldx_f = (float) oldx;
 	x = (int) make_long(tptr->down);
+	x_f = (float) x;
 	if (tptr->down->ttype == S_NUMBER)
 	  {
 	    tptr->res = tptr->down->res = ASINT;
@@ -693,6 +702,7 @@ int cubicconstcheck(tnode * tptr, tnode * nsl, int size, float ** ret)
 
   x = oldx = infl = oldinfl = -1;
   y = oldy = oldery = -1.0F;
+  x_f = oldx_f = infl_f = oldinfl_f = -1.0F;
 
   i = 1;
   tptr = nptr;
@@ -701,7 +711,9 @@ int cubicconstcheck(tnode * tptr, tnode * nsl, int size, float ** ret)
       switch (i & 3) {
       case 1:
 	oldinfl = infl;
+	oldinfl_f = (float) oldinfl;
 	infl = tptr->down->arrayidx;
+	infl_f = (float) infl;
 	if (tptr->next && tptr->next->next)
 	  {
 	    oldery = oldy;
@@ -710,31 +722,34 @@ int cubicconstcheck(tnode * tptr, tnode * nsl, int size, float ** ret)
 	  }
 	if (i > 1)
 	  {
-	    Q = oldinfl*oldinfl*oldinfl - x*x*x - 3.0F*x*x*(oldinfl-x);
-	    R = oldinfl*oldinfl - x*x - 2.0F*x*(oldinfl-x);
-	    S = x*x*x - infl*infl*infl - 3.0F*x*x*(x-infl);
-	    T = x*x - infl*infl - 2.0F*x*(x-infl);
+	    Q = (oldinfl_f*oldinfl_f*oldinfl_f - x_f*x_f*x_f
+		 - 3.0F*x_f*x_f*(oldinfl_f-x_f));
+	    R = oldinfl_f*oldinfl_f - x_f*x_f - 2.0F*x_f*(oldinfl_f-x_f);
+	    S = x_f*x_f*x_f - infl_f*infl_f*infl_f - 3.0F*x_f*x_f*(x_f-infl_f);
+	    T = x_f*x_f - infl_f*infl_f - 2.0F*x_f*(x_f-infl_f);
 
 	    a = 1.0F/((Q/R) - (S/T));
 	    a *= (oldery-oldy)/R - (oldy-y)/T;
 	    b = 1.0F/((R/Q) - (T/S));
 	    b *= (oldery-oldy)/Q - (oldy-y)/S;
-	    c = - 3.0F*a*x*x - 2.0F*b*x;
-	    d = oldy - a*x*x*x;
-	    d += - b*x*x - c*x;
+	    c = - 3.0F*a*x_f*x_f - 2.0F*b*x_f;
+	    d = oldy - a*x_f*x_f*x_f;
+	    d += - b*x_f*x_f - c*x_f;
     
 	    segptr = fptr + (infl - oldinfl);
 	    while ((fptr < endptr) && (fptr < segptr))
 	      {
-		*(fptr++) = oldinfl*oldinfl*oldinfl*a +
-		  oldinfl*oldinfl*b + oldinfl*c + d;
-		oldinfl++;
+		*(fptr++) = oldinfl_f*oldinfl_f*oldinfl_f*a +
+		  oldinfl_f*oldinfl_f*b + oldinfl_f*c + d;
+		oldinfl_f = (float) (++oldinfl);
 	      }
 	  }
 	break;
       case 3:
 	oldx = x;
+	oldx_f = (float) oldx;
 	x = tptr->down->arrayidx;
+	x_f = (float) x;
 	break;
       case 0:
 	oldery = oldy;
@@ -1440,7 +1455,7 @@ int splineconstcheck(tnode * tptr, tnode * nsl, int size, float ** ret)
   int i = 1;
   int x = -1;
   int oldx = -1;
-  float y, k, oldy, oldk;
+  float y, k, oldy, oldk, xf, oldxf;
   tnode * nptr = tptr;
   char newval[STRSIZE];
   float Q, R, S, a, b, c, d;
@@ -1477,15 +1492,18 @@ int splineconstcheck(tnode * tptr, tnode * nsl, int size, float ** ret)
 	tptr->down->arrayidx = x;
 	if ((i != 1) && (oldx != x))
 	  {
-	    Q = 1.0F/(oldx - x);
-	    R = Q*(oldx*oldx - x*x) - 2.0F*oldx;
+	    oldxf = (float) oldx;
+	    xf = (float) x;
+
+	    Q = 1.0F/(oldxf - xf);
+	    R = Q*(oldxf*oldxf - xf*xf) - 2.0F*oldxf;
 	    if (R == 0.0F)
 	      {     
 		printf("Error: No spline solution for these parameters.\n");
 		printtableproblem(tptr, nsl);
 	      }	 
 	    R =1.0F/R;
-	    S = Q*(oldx*oldx - x*x) - 2.0F*x;
+	    S = Q*(oldxf*oldxf - xf*xf) - 2.0F*xf;
 	    if (S == 0.0F)
 	      {     
 		printf("Error: No spline solution for these parameters.\n");
@@ -1493,8 +1511,8 @@ int splineconstcheck(tnode * tptr, tnode * nsl, int size, float ** ret)
 	      }	 
 	    S =1.0F/S;
 	    
-	    a = Q*(oldx*oldx*oldx-x*x*x)*(R-S);
-	    a += -3.0F*(R*oldx*oldx - S*x*x);
+	    a = Q*(oldxf*oldxf*oldxf-xf*xf*xf)*(R-S);
+	    a += -3.0F*(R*oldxf*oldxf - S*xf*xf);
 	    if (a==0.0F)
 	      {     
 		printf("Error: No spline solution for these parameters.\n");
@@ -1555,29 +1573,32 @@ int splineconstcheck(tnode * tptr, tnode * nsl, int size, float ** ret)
 	y = (float)atof(tptr->down->val);
 	if ((i > 2) && (oldx != x))
 	  {
+	    xf = (float) x;
+	    oldxf = (float) oldx;
+
 	    /* compute spline parameters */
 
-	    Q = 1.0F/(oldx - x);
-	    R = Q*(oldx*oldx - x*x) - 2.0F*oldx;
+	    Q = 1.0F/(oldxf - xf);
+	    R = Q*(oldxf*oldxf - xf*xf) - 2.0F*oldxf;
 	    R =1.0F/R;
-	    S = Q*(oldx*oldx - x*x) - 2.0F*x;
+	    S = Q*(oldxf*oldxf - xf*xf) - 2.0F*xf;
 	    S =1.0F/S;
-	    a = Q*(oldx*oldx*oldx-x*x*x)*(R-S);
-	    a += -3.0F*(R*oldx*oldx - S*x*x);
+	    a = Q*(oldxf*oldxf*oldxf-xf*xf*xf)*(R-S);
+	    a += -3.0F*(R*oldxf*oldxf - S*xf*xf);
 	    a =1.0F/a;
 	    a *= Q*(oldy-y)*(R-S) - R*oldk + S*k;
 	    b = Q*(oldk-k)*0.5F;
-	    b += - 1.5F*a*Q*(oldx*oldx - x*x);
-	    c = oldk - 3.0F*a*oldx*oldx -2.0F*b*oldx;
-	    d = oldy-a*oldx*oldx*oldx-b*oldx*oldx-c*oldx;
+	    b += - 1.5F*a*Q*(oldxf*oldxf - xf*xf);
+	    c = oldk - 3.0F*a*oldxf*oldxf -2.0F*b*oldxf;
+	    d = oldy-a*oldxf*oldxf*oldxf-b*oldxf*oldxf-c*oldxf;
 
 	    /* fill table */
 
 	    segptr = fptr + (x - oldx);
 	    while ((fptr < endptr) && (fptr < segptr))
 	      {
-		*(fptr++) = oldx*oldx*oldx*a + oldx*oldx*b + oldx*c + d;
-		oldx++;
+		*(fptr++) = oldxf*oldxf*oldxf*a + oldxf*oldxf*b + oldxf*c + d;
+		oldxf = (float) (++oldx);
 	      }
 	  }
 	break;
